# -*- coding: utf-8 -*-
"""Tests for base_resource module."""

import unittest

from requests import Response

from wiremock.base import RestClient
from wiremock.exceptions import UnexpectedResponseException
from wiremock.tests.base import BaseClientTestCase, attr


class RestClientTestCase(BaseClientTestCase):

    def setUp(self):
        super(RestClientTestCase, self).setUp()
        self.client = RestClient()

    @attr('unit')
    def test_handle_response(self):
        for status_code in [200, 201, 204]:
            resp = self._create_dummy_response(status_code)
            returned = self.client.handle_response(resp)
            self.assertEqual(returned, resp)

        resp = self._create_dummy_response(203)
        with self.assertRaises(UnexpectedResponseException):
            self.client.handle_response(resp)

    # Helpers

    def _create_dummy_response(self, status_code=200):
        resp = Response()
        resp.status_code = status_code
        return resp


if __name__ == '__main__':
    unittest.main()




class ApiException(Exception):
    """
    Generic Catch-all for API Exceptions.
    """

    status_code = None

    def __init__(self, status_code=None, msg=None, *args, **kwargs):
        self.status_code = status_code
        super(ApiException, self).__init__(msg, *args, **kwargs)



from __future__ import unicode_literals

import responses

from wiremock.tests.base import BaseClientTestCase, attr
from wiremock.client import Scenarios


class ScenariosResourceTests(BaseClientTestCase):

    @attr('unit', 'scenarios', 'resource')
    @responses.activate
    def test_reset_scenarios(self):
        responses.add(responses.POST, 'http://localhost/__admin/scenarios/reset', body='', status=200)

        r = Scenarios.reset_all_scenarios()
        self.assertEquals(200, r.status_code)



from .models import Mapping, MappingResponse, MappingRequest, DelayDistribution, ResponseFaultType, \
    DelayDistributionMethods, BasicAuthCredentials, WireMockMatchers, HttpMethods, CommonHeaders, \
    MappingMeta, AllMappings



from __future__ import unicode_literals

from wiremock.tests.base import BaseClientTestCase, attr
from wiremock.resources.settings import GlobalSetting


class SettingsSerializationTests(BaseClientTestCase):

    @attr('unit', 'serialization', 'settings')
    def test_global_settings_serialization(self):
        gs = GlobalSetting(fixed_delay=500)
        serialized = gs.get_json_data()
        self.assertDictContainsKeyWithValue(serialized, 'fixedDelay', 500)

    @attr('unit', 'serialization', 'settings')
    def test_global_settings_deserialization(self):
        serialized = {'fixedDelay': 500}
        gs = GlobalSetting.from_dict(serialized)
        self.assertIsInstance(gs, GlobalSetting)
        self.assertEquals(500, gs.fixed_delay)



# import Exceptions
from .exceptions import *

# import Models
from .resources.settings import GlobalSetting
from .resources.mappings import Mapping, MappingResponse, MappingRequest, DelayDistribution,\
    ResponseFaultType, DelayDistributionMethods, BasicAuthCredentials, WireMockMatchers, HttpMethods, \
    CommonHeaders, MappingMeta, AllMappings
from .resources.requests import RequestResponse, RequestResponseDefinition, RequestResponseRequest, \
    RequestCountResponse, RequestResponseAll, RequestResponseFindResponse, RequestResponseAllMeta
from .resources.near_misses import NearMissMatchResponse, NearMissMatchRequest, NearMissMatchResult, \
    NearMissRequestPatternResult, NearMissMatch, NearMissMatchPatternRequest

# import Resources
from .resources.settings.resource import GlobalSettings
from .resources.mappings.resource import Mappings
from .resources.requests.resource import Requests
from .resources.near_misses.resource import NearMisses
from .resources.scenarios.resource import Scenarios



from __future__ import unicode_literals

from wiremock.resources.mappings import BasicAuthCredentials
from wiremock.tests.base import BaseClientTestCase, attr
from wiremock.resources.requests import RequestResponse, RequestCountResponse, RequestResponseDefinition, \
    RequestResponseAll, RequestResponseFindResponse, RequestResponseRequest, RequestResponseAllMeta


class RequestsSerializationTests(BaseClientTestCase):

    @attr('unit', 'serialization', 'requests')
    def test_request_count_response_serialization(self):
        e = RequestCountResponse(
            count=1
        )
        serialized = e.get_json_data()
        self.assertDictContainsKeyWithValue(serialized, 'count', 1)

    @attr('unit', 'serialization', 'requests')
    def test_request_count_response_deserialization(self):
        serialized = {'count': 1}
        e = RequestCountResponse.from_dict(serialized)
        self.assertIsInstance(e, RequestCountResponse)
        self.assertEquals(1, e.count)

    @attr('unit', 'serialization', 'requests')
    def test_request_response_all_meta_serialization(self):
        e = RequestResponseAllMeta(
            total=1
        )
        serialized = e.get_json_data()
        self.assertDictContainsKeyWithValue(serialized, 'total', 1)

    @attr('unit', 'serialization', 'requests')
    def test_request_response_all_meta_deserialization(self):
        serialized = {'total': 1}
        e = RequestResponseAllMeta.from_dict(serialized)
        self.assertIsInstance(e, RequestResponseAllMeta)
        self.assertEquals(1, e.total)

    @attr('unit', 'serialization', 'requests')
    def test_request_response_request_serialization(self):
        e = RequestResponseRequest(
            method='GET', url='test', absolute_url='test2', client_ip='test3',
            basic_auth_credentials=BasicAuthCredentials(username='username', password='password'),
            cookies={'chocolate': 'chip'}, headers={'test': '1'}, query_parameters={'test2': '2'},
            browser_proxy_request=False, body='test4', body_as_base64='test5', logged_date=12345,
            logged_date_string='test6'
        )
        serialized = e.get_json_data()
        self.assertDictContainsKeyWithValue(serialized, 'method', 'GET')
        self.assertDictContainsKeyWithValue(serialized, 'url', 'test')
        self.assertDictContainsKeyWithValue(serialized, 'absoluteUrl', 'test2')
        self.assertDictContainsKeyWithValue(serialized, 'clientIp', 'test3')
        self.assertDictContainsKeyWithValue(serialized, 'basicAuthCredentials', {'username': 'username', 'password': 'password'})
        self.assertDictContainsKeyWithValue(serialized, 'cookies', {'chocolate': 'chip'})
        self.assertDictContainsKeyWithValue(serialized, 'headers', {'test': '1'})
        self.assertDictContainsKeyWithValue(serialized, 'queryParameters', {'test2': '2'})
        self.assertDictContainsKeyWithValue(serialized, 'browserProxyRequest', False)
        self.assertDictContainsKeyWithValue(serialized, 'body', 'test4')
        self.assertDictContainsKeyWithValue(serialized, 'bodyAsBase64', 'test5')
        self.assertDictContainsKeyWithValue(serialized, 'loggedDate', 12345)
        self.assertDictContainsKeyWithValue(serialized, 'loggedDateString', 'test6')

    @attr('unit', 'serialization', 'requests')
    def test_request_response_request_deserialization(self):
        serialized = {
            'method': 'GET', 'url': 'test', 'absoluteUrl': 'test2', 'clientIp': 'test3',
            'basicAuthCredentials': {'username': 'username', 'password': 'password'},
            'cookies': {'chocolate': 'chip'}, 'headers': {'test': '1'},
            'queryParameters': {'test2': '2'}, 'browserProxyRequest': False,
            'body': 'test4', 'bodyAsBase64': 'test5', 'loggedDate': 12345, 'loggedDateString': 'test6'
        }
        e = RequestResponseRequest.from_dict(serialized)
        self.assertIsInstance(e, RequestResponseRequest)
        self.assertEquals('GET', e.method)
        self.assertEquals('test', e.url)
        self.assertEquals('test2', e.absolute_url)
        self.assertEquals('test3', e.client_ip)
        self.assertIsInstance(e.basic_auth_credentials, BasicAuthCredentials)
        self.assertEquals('username', e.basic_auth_credentials.username)
        self.assertEquals('password', e.basic_auth_credentials.password)
        self.assertEquals({'chocolate': 'chip'}, e.cookies)
        self.assertEquals({'test': '1'}, e.headers)
        self.assertEquals({'test2': '2'}, e.query_parameters)
        self.assertEquals(False, e.browser_proxy_request)
        self.assertEquals('test4', e.body)
        self.assertEquals('test5', e.body_as_base64)
        self.assertEquals(12345, e.logged_date)
        self.assertEquals('test6', e.logged_date_string)

    @attr('unit', 'serialization', 'requests')
    def test_request_response_definition_serialization(self):
        e = RequestResponseDefinition(
            status=200, transformers=['test'], from_configured_stub=False, transformer_parameters={'test2': '2'}
        )
        serialized = e.get_json_data()
        self.assertDictContainsKeyWithValue(serialized, 'status', 200)
        self.assertDictContainsKeyWithValue(serialized, 'transformers', ['test'])
        self.assertDictContainsKeyWithValue(serialized, 'fromConfiguredStub', False)
        self.assertDictContainsKeyWithValue(serialized, 'transformerParameters', {'test2': '2'})

    @attr('unit', 'serialization', 'requests')
    def test_request_response_definition_deserialization(self):
        serialized = {
            'status': 200, 'transformers': ['test'], 'fromConfiguredStub': False,
            'transformerParameters': {'test2': '2'}
        }
        e = RequestResponseDefinition.from_dict(serialized)
        self.assertIsInstance(e, RequestResponseDefinition)
        self.assertEquals(200, e.status)
        self.assertEquals(['test'], e.transformers)
        self.assertEquals(False, e.from_configured_stub)
        self.assertEquals({'test2': '2'}, e.transformer_parameters)

    @attr('unit', 'serialization', 'requests')
    def test_request_response_serialization(self):
        e = RequestResponse(
            request=RequestResponseRequest(method='GET', url='test'),
            response_definition=RequestResponseDefinition(status=200)
        )
        serialized = e.get_json_data()
        self.assertDictContainsKeyWithValue(serialized, 'request', {'method': 'GET', 'url': 'test'})
        self.assertDictContainsKeyWithValue(serialized, 'responseDefinition', {'status': 200})

    @attr('unit', 'serialization', 'requests')
    def test_request_response_deserialization(self):
        serialized = {'request': {'method': 'GET', 'url': 'test'}, 'responseDefinition': {'status': 200}}
        e = RequestResponse.from_dict(serialized)
        self.assertIsInstance(e, RequestResponse)
        self.assertIsInstance(e.request, RequestResponseRequest)
        self.assertEquals('GET', e.request.method)
        self.assertEquals('test', e.request.url)
        self.assertIsInstance(e.response_definition, RequestResponseDefinition)
        self.assertEquals(200, e.response_definition.status)

    @attr('unit', 'serialization', 'requests')
    def test_request_response_find_response_serialization(self):
        e = RequestResponseFindResponse(
            requests=[RequestResponseRequest(method='GET', url='test'), ]
        )
        serialized = e.get_json_data()
        self.assertDictContainsKeyWithValueType(serialized, 'requests', list)
        self.assertDictContainsKeyWithValue(serialized, 'requests', [{'method': 'GET', 'url': 'test'}, ])

    @attr('unit', 'serialization', 'requests')
    def test_request_response_find_response_deserialization(self):
        serialized = {'requests': [{'method': 'GET', 'url': 'test'}, ]}
        e = RequestResponseFindResponse.from_dict(serialized)
        self.assertIsInstance(e, RequestResponseFindResponse)
        self.assertIsInstance(e.requests, list)
        self.assertIsInstance(e.requests[0], RequestResponseRequest)
        self.assertEquals('GET', e.requests[0].method)
        self.assertEquals('test', e.requests[0].url)

    @attr('unit', 'serialization', 'requests')
    def test_request_response_all_serialization(self):
        e = RequestResponseAll(
            requests=[RequestResponse(
                request=RequestResponseRequest(method='GET', url='test'),
                response_definition=RequestResponseDefinition(status=200)
            ), ],
            meta=RequestResponseAllMeta(total=1),
            request_journal_disabled=False
        )
        serialized = e.get_json_data()
        self.assertDictContainsKeyWithValue(serialized, 'requestJournalDisabled', False)
        self.assertDictContainsKeyWithValue(serialized, 'requests', [
            {'request': {'method': 'GET', 'url': 'test'}, 'responseDefinition': {'status': 200}},
        ])
        self.assertDictContainsKeyWithValue(serialized, 'meta', {'total': 1})

    @attr('unit', 'serialization', 'requests')
    def test_request_response_all_deserialization(self):
        serialized = {
            'requests': [
                {'request': {'method': 'GET', 'url': 'test'}, 'responseDefinition': {'status': 200}},
            ],
            'meta': {'total': 1},
            'requestJournalDisabled': False
        }
        e = RequestResponseAll.from_dict(serialized)
        self.assertIsInstance(e, RequestResponseAll)
        self.assertEquals(False, e.request_journal_disabled)
        self.assertIsInstance(e.requests, list)
        rr = e.requests[0]
        self.assertIsInstance(rr, RequestResponse)
        self.assertIsInstance(rr.request, RequestResponseRequest)
        self.assertEquals('GET', rr.request.method)
        self.assertEquals('test', rr.request.url)
        self.assertIsInstance(rr.response_definition, RequestResponseDefinition)
        self.assertEquals(200, rr.response_definition.status)
        self.assertIsInstance(e.meta, RequestResponseAllMeta)
        self.assertEquals(1, e.meta.total)



from __future__ import unicode_literals
import json
import logging
from nose.tools import nottest
from nose.plugins.attrib import attr
import responses
from unittest import TestCase

from wiremock.constants import Config


logging.basicConfig(level=logging.DEBUG)


class BaseClientTestCase(TestCase):
    """
    Adds utility methods.
    """

    @classmethod
    def setUpClass(cls):
        Config.base_url = 'http://localhost/__admin'
        Config.timeout = 1
        super(BaseClientTestCase, cls).setUpClass()

    def assertHasAttr(self, obj, attr):
        self.assertTrue(hasattr(obj, attr), "%s doesn't have attribute: %s" % (obj, attr))

    def assertNotHasAttr(self, obj, attr):
        self.assertFalse(hasattr(obj, attr), "%s shouldn't have attribute: %s" % (obj, attr))

    def assertAttrEqual(self, obj, attr, value):
        self.assertHasAttr(obj, attr)
        self.assertEqual(getattr(obj, attr), value)

    def assertAttrNotEqual(self, obj, attr, value):
        self.assertHasAttr(obj, attr)
        self.assertNotEqual(getattr(obj, attr), value)

    def assertNotRaise(self, callableObj, *args, **kwargs):
        try:
            callableObj(*args, **kwargs)
        except Exception as e:
            raise AssertionError("Shouldn't raise and exception: {}".format(e))

    def assertAnyRaise(self, callableObj, *args, **kwargs):
        try:
            callableObj(*args, **kwargs)
        except:
            return
        raise AssertionError("Should raise an exception")

    def assertIsSubclass(self, C, B):
        if issubclass(C, B):
            return
        else:
            raise AssertionError("{} is Not a Subclass of {}".format(B, C))

    def assertDictContainsKey(self, obj, key):
        if key in obj:
            return
        else:
            raise AssertionError("{} is not in dict: {}".format(key, obj))

    def assertDictContainsKeyWithValue(self, obj, key, value):
        if key in obj:
            self.assertEquals(value, obj[key])
        else:
            raise AssertionError("{} is not in dict: {}".format(key, obj))

    def assertDictContainsKeyWithValueType(self, obj, key, B):
        if key in obj:
            if isinstance(obj[key], B):
                return
            else:
                raise AssertionError("dict[{}]={} is not of type: {}".format(key, obj[key], B))
        else:
            raise AssertionError("{} is not in dict: {}".format(key, obj))


__all__ = ['BaseClientTestCase', 'nottest', 'attr', 'json', 'responses']



from __future__ import unicode_literals

from wiremock.constants import make_headers
from wiremock.base.base_resource import BaseResource
from wiremock.resources.mappings import Mapping, AllMappings, MappingResponse


class Mappings(BaseResource):

    @classmethod
    def endpoint(cls):
        return '/mappings'

    @classmethod
    def endpoint_single(cls):
        return '/mappings/{id}'

    @classmethod
    def entity_class(cls):
        return MappingResponse

    @classmethod
    def create_mapping(cls, mapping, parameters={}):
        cls.validate_is_entity(mapping, Mapping)
        response = cls.REST_CLIENT.post(
            cls.get_base_uri(cls.endpoint()),
            json=mapping.get_json_data(),
            headers=make_headers(),
            params=parameters
        )
        response = cls.REST_CLIENT.handle_response(response)
        return MappingResponse.from_dict(response.json())

    @classmethod
    def retrieve_all_mappings(cls, parameters={}):
        response = cls.REST_CLIENT.get(
            cls.get_base_uri(cls.endpoint()),
            headers=make_headers(),
            params=parameters
        )
        response = cls.REST_CLIENT.handle_response(response)
        return AllMappings.from_dict(response.json())

    @classmethod
    def retrieve_mapping(cls, mapping_id, parameters={}):
        mapping_id = cls.get_entity_id(mapping_id, Mapping)
        ids = {'id': mapping_id}
        response = cls.REST_CLIENT.get(
            cls.get_base_uri(cls.endpoint_single(), **ids),
            headers=make_headers(),
            params=parameters
        )
        response = cls.REST_CLIENT.handle_response(response)
        return Mapping.from_dict(response.json())

    @classmethod
    def update_mapping(cls, mapping, parameters={}):
        cls.validate_is_entity(mapping, Mapping)
        mapping_id = cls.get_entity_id(mapping, Mapping)
        ids = {'id': mapping_id}
        response = cls.REST_CLIENT.put(
            cls.get_base_uri(cls.endpoint_single(), **ids),
            json=mapping.get_json_data(),
            headers=make_headers(),
            params=parameters
        )
        response = cls.REST_CLIENT.handle_response(response)
        return Mapping.from_dict(response.json())

    @classmethod
    def persist_mappings(cls, parameters={}):
        ids = {'id': 'save'}
        response = cls.REST_CLIENT.post(
            cls.get_base_uri(cls.endpoint_single(), **ids),
            headers=make_headers(),
            params=parameters
        )
        return cls.REST_CLIENT.handle_response(response)

    @classmethod
    def reset_mappings(cls, parameters={}):
        ids = {'id': 'reset'}
        response = cls.REST_CLIENT.post(
            cls.get_base_uri(cls.endpoint_single(), **ids),
            headers=make_headers(),
            params=parameters
        )
        return cls.REST_CLIENT.handle_response(response)

    @classmethod
    def delete_all_mappings(cls, parameters={}):
        response = cls.REST_CLIENT.delete(
            cls.get_base_uri(cls.endpoint()),
            headers=make_headers(),
            params=parameters
        )
        return cls.REST_CLIENT.handle_response(response)

    @classmethod
    def delete_mapping(cls, mapping_id, parameters={}):
        mapping_id = cls.get_entity_id(mapping_id, Mapping)
        ids = {'id': mapping_id}
        response = cls.REST_CLIENT.delete(
            cls.get_base_uri(cls.endpoint_single(), **ids),
            headers=make_headers(),
            params=parameters
        )
        return cls.REST_CLIENT.handle_response(response)


__all__ = ['Mappings']



from __future__ import unicode_literals

from wiremock._compat import add_metaclass
from wiremock.base import JsonProperty, BaseAbstractEntity, BaseEntityMetaType
from wiremock.resources.mappings.models import BasicAuthCredentials


@add_metaclass(BaseEntityMetaType)
class NearMissMatchRequest(BaseAbstractEntity):
    url = JsonProperty('url')
    absolute_url = JsonProperty('absoluteUrl')
    method = JsonProperty('method')
    client_ip = JsonProperty('clientIp')
    headers = JsonProperty('headers', klass=dict)
    query_parameters = JsonProperty('queryParameters', klass=dict)
    cookies = JsonProperty('cookies', klass=dict)
    basic_auth_credentials = JsonProperty('basicAuthCredentials', klass=BasicAuthCredentials)
    browser_proxy_request = JsonProperty('browserProxyRequest')  # type: bool
    body_as_base64 = JsonProperty('bodyAsBase64')
    body = JsonProperty('body')
    logged_date = JsonProperty('loggedDate')  # epoch seconds
    logged_date_string = JsonProperty('loggedDateString')


@add_metaclass(BaseEntityMetaType)
class NearMissMatchPatternRequest(BaseAbstractEntity):
    url = JsonProperty('url')
    url_pattern = JsonProperty('urlPattern')
    url_path = JsonProperty('urlPath')
    url_path_pattern = JsonProperty('urlPathPattern')
    method = JsonProperty('method')
    client_ip = JsonProperty('clientIp')
    headers = JsonProperty('headers', klass=dict)
    query_parameters = JsonProperty('queryParameters', klass=dict)
    cookies = JsonProperty('cookies', klass=dict)
    body_patterns = JsonProperty('bodyPatterns', klass=dict)
    basic_auth_credentials = JsonProperty('basicAuthCredentials', klass=BasicAuthCredentials)
    browser_proxy_request = JsonProperty('browserProxyRequest')  # type: bool
    logged_date = JsonProperty('loggedDate')  # epoch seconds
    logged_date_string = JsonProperty('loggedDateString')


# Responses

@add_metaclass(BaseEntityMetaType)
class NearMissRequestPatternResult(BaseAbstractEntity):
    url = JsonProperty('url')
    absolute_url = JsonProperty('absoluteUrl')
    method = JsonProperty('method')
    client_ip = JsonProperty('clientIp')
    headers = JsonProperty('headers', klass=dict)
    query_parameters = JsonProperty('queryParameters', klass=dict)
    cookies = JsonProperty('cookies', klass=dict)
    basic_auth_credentials = JsonProperty('basicAuthCredentials', klass=BasicAuthCredentials)
    browser_proxy_request = JsonProperty('browserProxyRequest')  # type: bool
    body_as_base64 = JsonProperty('bodyAsBase64')
    body = JsonProperty('body')


@add_metaclass(BaseEntityMetaType)
class NearMissMatchResult(BaseAbstractEntity):
    distance = JsonProperty('distance')  # type: float


@add_metaclass(BaseEntityMetaType)
class NearMissMatch(BaseAbstractEntity):
    request = JsonProperty('request', klass=NearMissMatchRequest)
    request_pattern = JsonProperty('requestPattern', klass=NearMissRequestPatternResult)
    match_result = JsonProperty('matchResult', klass=NearMissMatchResult)


@add_metaclass(BaseEntityMetaType)
class NearMissMatchResponse(BaseAbstractEntity):
    near_misses = JsonProperty('nearMisses', klass=list, list_klass=NearMissMatch)


__all__ = ['NearMissMatchResponse', 'NearMissMatchRequest', 'NearMissMatchResult', 'NearMissRequestPatternResult',
           'NearMissMatch', 'NearMissMatchPatternRequest']



from __future__ import unicode_literals

from wiremock._compat import add_metaclass
from wiremock.base import JsonProperty, BaseAbstractEntity, BaseEntityMetaType


@add_metaclass(BaseEntityMetaType)
class GlobalSetting(BaseAbstractEntity):
    fixed_delay = JsonProperty('fixedDelay')


__all__ = ['GlobalSetting']



from __future__ import unicode_literals

from wiremock._compat import add_metaclass
from wiremock.base import BaseEntity, JsonProperty, BaseAbstractEntity, BaseEntityMetaType
from wiremock.resources.mappings.models import BasicAuthCredentials


@add_metaclass(BaseEntityMetaType)
class RequestCountResponse(BaseAbstractEntity):
    count = JsonProperty('count')


@add_metaclass(BaseEntityMetaType)
class RequestResponseRequest(BaseAbstractEntity):
    method = JsonProperty('method')
    url = JsonProperty('url')
    absolute_url = JsonProperty('absoluteUrl')
    client_ip = JsonProperty('clientIp')
    basic_auth_credentials = JsonProperty('basicAuthCredentials', klass=BasicAuthCredentials)
    cookies = JsonProperty('cookies', klass=dict)
    headers = JsonProperty('headers', klass=dict)
    query_parameters = JsonProperty('queryParameters', klass=dict)
    browser_proxy_request = JsonProperty('browserProxyRequest')  # should be true/false
    body = JsonProperty('body')
    body_as_base64 = JsonProperty('bodyAsBase64')
    logged_date = JsonProperty('loggedDate')  # epoch seconds
    logged_date_string = JsonProperty('loggedDateString')


@add_metaclass(BaseEntityMetaType)
class RequestResponseDefinition(BaseAbstractEntity):
    status = JsonProperty('status')
    transformers = JsonProperty('transformers', klass=list)
    from_configured_stub = JsonProperty('fromConfiguredStub')  # will be true/false
    transformer_parameters = JsonProperty('transformerParameters', klass=dict)


class RequestResponse(BaseEntity):
    request = JsonProperty('request', klass=RequestResponseRequest)
    response_definition = JsonProperty('responseDefinition', klass=RequestResponseDefinition)


@add_metaclass(BaseEntityMetaType)
class RequestResponseAllMeta(BaseAbstractEntity):
    total = JsonProperty('total')


@add_metaclass(BaseEntityMetaType)
class RequestResponseFindResponse(BaseAbstractEntity):
    requests = JsonProperty('requests', klass=list, list_klass=RequestResponseRequest)


@add_metaclass(BaseEntityMetaType)
class RequestResponseAll(BaseAbstractEntity):
    requests = JsonProperty('requests', klass=list, list_klass=RequestResponse)
    meta = JsonProperty('meta', klass=RequestResponseAllMeta)
    request_journal_disabled = JsonProperty('requestJournalDisabled')  # should be true/false


__all__ = ['RequestResponse', 'RequestResponseDefinition', 'RequestResponseRequest', 'RequestCountResponse',
           'RequestResponseAll', 'RequestResponseFindResponse', 'RequestResponseAllMeta']



from __future__ import unicode_literals

from wiremock.constants import make_headers
from wiremock.base.base_resource import BaseResource
from wiremock.resources.requests import RequestCountResponse, RequestResponse, RequestResponseAll, \
    RequestResponseFindResponse
from wiremock.resources.near_misses import NearMissMatchPatternRequest, NearMissMatchResponse


class Requests(BaseResource):

    @classmethod
    def endpoint(cls):
        return '/requests'

    @classmethod
    def endpoint_single(cls):
        return '/requests/{id}'

    @classmethod
    def endpoint_requests_unmatched(cls):
        return cls.endpoint() + '/unmatched'

    @classmethod
    def endpoint_request_unmatched_near_misses(cls):
        return cls.endpoint_requests_unmatched() + '/near-misses'

    @classmethod
    def entity_class(cls):
        return RequestResponse

    @classmethod
    def get_all_received_requests(cls, limit=None, since=None, parameters={}):
        if limit is not None and limit > 0:
            parameters['limit'] = limit
        if since is not None:
            parameters['since'] = since

        response = cls.REST_CLIENT.get(cls.endpoint(), params=parameters, headers=make_headers())
        response = cls.REST_CLIENT.handle_response(response)

        return RequestResponseAll.from_dict(response.json())

    @classmethod
    def get_request(cls, request_id, parameters={}):
        ids = {'id': request_id}
        response = cls.REST_CLIENT.get(
            cls.get_base_uri(cls.endpoint_single(), **ids),
            headers=make_headers(),
            params=parameters
        )
        response = cls.REST_CLIENT.handle_response(response)
        return RequestResponse.from_dict(response.json())

    @classmethod
    def reset_request_journal(cls, parameters={}):
        ids = {'id': 'reset'}
        response = cls.REST_CLIENT.post(
            cls.get_base_uri(cls.endpoint_single(), **ids),
            headers=make_headers(),
            params=parameters
        )
        return cls.REST_CLIENT.handle_response(response)

    @classmethod
    def get_matching_request_count(cls, request, parameters={}):
        cls.validate_is_entity(request, NearMissMatchPatternRequest)
        ids = {'id': 'count'}
        response = cls.REST_CLIENT.post(
            cls.get_base_uri(cls.endpoint_single(), **ids),
            json=request.get_json_data(),
            headers=make_headers(),
            params=parameters
        )
        response = cls.REST_CLIENT.handle_response(response)
        return RequestCountResponse.from_dict(response.json())

    @classmethod
    def get_matching_requests(cls, request, parameters={}):
        cls.validate_is_entity(request, NearMissMatchPatternRequest)
        ids = {'id': 'find'}
        response = cls.REST_CLIENT.post(
            cls.get_base_uri(cls.endpoint_single(), **ids),
            json=request.get_json_data(),
            headers=make_headers(),
            params=parameters
        )
        response = cls.REST_CLIENT.handle_response(response)
        return RequestResponseFindResponse.from_dict(response.json())

    @classmethod
    def get_unmatched_requests(cls, parameters={}):
        response = cls.REST_CLIENT.get(
            cls.get_base_uri(cls.endpoint_requests_unmatched()),
            headers=make_headers(),
            params=parameters
        )
        response = cls.REST_CLIENT.handle_response(response)

        return RequestResponseFindResponse.from_dict(response.json())

    @classmethod
    def get_unmatched_requests_near_misses(cls, parameters={}):
        response = cls.REST_CLIENT.get(
            cls.get_base_uri(cls.endpoint_request_unmatched_near_misses()),
            headers=make_headers(),
            params=parameters
        )
        response = cls.REST_CLIENT.handle_response(response)
        return NearMissMatchResponse.from_dict(response.json())


__all__ = ['Requests']



from __future__ import unicode_literals

from wiremock.base.base_resource import BaseResource
from wiremock.resources.near_misses import NearMissMatchPatternRequest, NearMissMatchRequest, NearMissMatchResponse


class NearMisses(BaseResource):

    @classmethod
    def endpoint(cls):
        return '/near-misses'

    @classmethod
    def endpoint_single(cls):
        return '/near-misses'

    @classmethod
    def endpoint_request(cls):
        return cls.endpoint() + '/request'

    @classmethod
    def endpoint_request_pattern(cls):
        return cls.endpoint() + '/request-pattern'

    @classmethod
    def entity_class(cls):
        return NearMissMatchResponse

    @classmethod
    def find_nearest_misses_by_request(cls, request, parameters={}):
        cls.validate_is_entity(request, NearMissMatchRequest)
        response = cls.REST_CLIENT.post(
            cls.get_base_uri(cls.endpoint_request()),
            json=request.get_json_data(),
            params=parameters
        )

        response = cls.REST_CLIENT.handle_response(response)
        return NearMissMatchResponse.from_dict(response.json())

    @classmethod
    def find_nearest_misses_by_request_pattern(cls, request_pattern, parameters={}):
        cls.validate_is_entity(request_pattern, NearMissMatchPatternRequest)
        response = cls.REST_CLIENT.post(
            cls.get_base_uri(cls.endpoint_request_pattern()),
            json=request_pattern.get_json_data(),
            params=parameters
        )

        response = cls.REST_CLIENT.handle_response(response)
        return NearMissMatchResponse.from_dict(response.json())


__all__ = ['NearMisses']



# -*- coding: utf-8 -*-
"""Test wiremock.server."""

import unittest
from subprocess import STDOUT, PIPE

from mock import patch, DEFAULT
from pkg_resources import resource_filename

from wiremock.server.exceptions import (
    WireMockServerAlreadyStartedError,
    WireMockServerNotStartedError
)
from wiremock.server.server import WireMockServer
from wiremock.tests.base import attr


class WireMockServerTestCase(unittest.TestCase):

    def setUp(self):
        self.java_path = '/path/to/java'
        self.jar_path = '/path/to/jar'
        self.port = 54321
        with patch.object(WireMockServer, '_get_free_port', return_value=self.port):
            self.wm = WireMockServer(
                java_path=self.java_path,
                jar_path=self.jar_path
            )

    @attr('unit', 'server')
    def test_init(self):
        with patch.object(WireMockServer, '_get_free_port') as _get_free_port:
            _get_free_port.return_value = self.port

            wm = WireMockServer(java_path=self.java_path, jar_path=self.jar_path)

            self.assertEqual(wm.port, _get_free_port.return_value)

        self.assertEqual(wm.java_path, self.java_path)
        self.assertEqual(wm.jar_path, self.jar_path)
        self.assertFalse(wm.is_running)

    @attr('unit', 'server')
    def test_init_with_defaults(self):
        with patch.object(WireMockServer, '_get_free_port', return_value=self.port):
            wm = WireMockServer()

        expected_jar = resource_filename(
            'wiremock',
            'server/wiremock-standalone-2.6.0.jar'
        )
        self.assertEqual(wm.java_path, 'java')  # Assume java in PATH
        self.assertEqual(wm.jar_path, expected_jar)

    @attr('unit', 'server')
    @patch('wiremock.server.server.socket')
    def test_get_free_port(self, mock_socket):
        sock = mock_socket.socket.return_value
        expected_port = 54321
        sock.getsockname.return_value = ('localhost', expected_port)

        port = self.wm._get_free_port()

        self.assertEqual(port, expected_port)

    @attr('unit', 'server')
    @patch('wiremock.server.server.atexit')
    @patch('wiremock.server.server.Popen')
    def test_start(self, Popen, atexit):

        def poll():
            Popen.return_value.returncode = None
            return None

        Popen.return_value.poll.side_effect = poll

        self.wm.start()

        Popen.assert_called_once_with(
            [self.java_path, '-jar', self.jar_path, '--port', str(54321)],
            stdin=PIPE,
            stdout=PIPE,
            stderr=STDOUT
        )

        self.assertTrue(self.wm.is_running)
        atexit.register.assert_called_once_with(self.wm.stop, raise_on_error=False)

        # Test when already started
        with self.assertRaises(WireMockServerAlreadyStartedError):
            self.wm.start()

    @attr('unit', 'server')
    def test_start_with_invalid_java(self):
        wm = WireMockServer(java_path='/no/such/path')
        with self.assertRaises(WireMockServerNotStartedError):
            wm.start()

    @attr('unit', 'server')
    def test_start_with_invalid_jar(self):
        wm = WireMockServer(jar_path='/dev/null')
        with self.assertRaises(WireMockServerNotStartedError):
            wm.start()

    @attr('unit', 'server')
    def test_stop(self):
        with patch.object(self.wm, '_WireMockServer__subprocess') as _subprocess:
            self.wm._WireMockServer__running = True

            self.wm.stop()

            _subprocess.kill.assert_called_once_with()

            # Test repeated call

            _subprocess.kill.side_effect = AttributeError
            with self.assertRaises(WireMockServerNotStartedError):
                self.wm.stop()

    @attr('unit', 'server')
    def test_with_statement(self):
        with patch.multiple(WireMockServer, start=DEFAULT, stop=DEFAULT) as mocks:

            with WireMockServer() as wm:
                self.assertIsInstance(wm, WireMockServer)
                mocks['start'].assert_called_once_with()

            mocks['stop'].assert_called_once_with()


if __name__ == '__main__':
    unittest.main()



from __future__ import unicode_literals
from collections import OrderedDict
import json

from wiremock._compat import add_metaclass, array_types


class JsonPropertyValueContainer(object):
    def __init__(self, json_property, value):
        self.json_property = json_property
        self.value = value

    def getval(self):
        return self.value

    def setval(self, value):
        self.value = value

    def delval(self):  # pragma: no cover
        self.value = None

    def get_property(self):  # pragma: no cover
        _get = lambda slf: self.getval()
        _set = lambda slf, val: self.setval(val)
        _del = lambda slf: self.delval()

        return property(_get, _set, _del)


class JsonProperty(object):

    value_container = JsonPropertyValueContainer

    def __init__(self, json_name, property_name=None,
                 klass=None, list_klass=None, dict_key_klass=None, dict_value_klass=None,
                 include_if_null=False):
        self._json_name = json_name
        self._property_name = property_name
        self._klass = klass

        if self._klass is not None and not issubclass(self._klass, BaseAbstractEntity):
            if issubclass(self._klass, list) or issubclass(self._klass, tuple):
                self._list_klass = list_klass
            else:
                self._list_klass = None

            if issubclass(self._klass, dict):
                self._dict_key_klass = dict_key_klass
                self._dict_value_klass = dict_value_klass
            else:
                self._dict_key_klass = None
                self._dict_value_klass = None
        else:
            self._list_klass = None
            self._dict_key_klass = None
            self._dict_value_klass = None

        self._include_if_null = include_if_null

    def set_property_name(self, property_name):
        self._property_name = property_name
        if self._json_name is None:  # pragma: no cover
            self._json_name = self._property_name

    @property
    def json_name(self):
        return self._json_name

    @property
    def property_name(self):
        return self._property_name

    @property
    def klass(self):
        return self._klass

    @property
    def list_klass(self):
        return self._list_klass

    @property
    def dict_key_klass(self):
        return self._dict_key_klass

    @property
    def dict_value_klass(self):
        return self._dict_value_klass

    def is_dict(self):
        return self._klass is not None and issubclass(self._klass, dict)

    def is_list(self):
        return self._klass is not None and (issubclass(self._klass, list) or issubclass(self._klass, tuple))

    def is_base_entity_class(self):
        return self._klass is not None and issubclass(self._klass, BaseAbstractEntity)

    @property
    def include_if_null(self):
        return self._include_if_null

    def __str__(self):
        return "JsonProperty(json_name={}, property_name={}, klass={}, include_if_null={})".format(
            self._json_name, self._property_name, self._klass, self._include_if_null
        )

    def __unicode__(self):
        return "JsonProperty(json_name={}, property_name={}, klass={}, include_if_null={})".format(
            self._json_name, self._property_name, self._klass, self._include_if_null
        )


class EntityModelException(Exception):
    pass


class BaseAbstractEntity(object):
    """
    The base abstract entity class, don't inherit from this, inherit from BaseEntity, defined below.
    """

    def __init__(self, **values):
        self._values = {}
        for name, prop in self._properties.items():
            value = values.get(prop.json_name, values.get(name, None))
            if prop.is_list() and isinstance(value, array_types):  # This is a list with sub types
                l = prop.klass()
                for v in value:
                    if prop.list_klass is not None and issubclass(prop.list_klass, BaseAbstractEntity):
                        l.append(prop.list_klass.from_dict(v))
                    else:
                        l.append(v)
                value = l
                value_container = prop.value_container(prop, l)
            elif prop.is_dict() and isinstance(value, dict):  # This is a dict with sub types
                d = {}
                for k, v in value.items():
                    rk = k
                    rv = v

                    if prop.dict_key_klass is None:
                        pass
                    elif issubclass(prop.dict_key_klass, BaseAbstractEntity):
                        rk = prop.dict_key_klass.from_dict(k)
                    else:
                        rk = prop.dict_key_klass(k)

                    if prop.dict_value_klass is None:
                        pass
                    elif issubclass(prop.dict_value_klass, BaseAbstractEntity):
                        rv = prop.dict_value_klass.from_dict(v)
                    else:
                        rv = prop.dict_value_klass(v)

                    d[rk] = rv
                value = d
                value_container = prop.value_container(prop, d)
            elif prop.is_base_entity_class() and isinstance(value, dict):  # This is a BaseAbstractEntity Class
                value = prop.klass.from_dict(value)
                value_container = prop.value_container(prop, value)
            else:
                value_container = prop.value_container(prop, value)
            self._values[name] = value_container
            setattr(self, name, value)

    def __eq__(self, other):
        if not isinstance(other, BaseAbstractEntity):  # pragma: no cover
            return False

        return self.get_json_data() == other.get_json_data()

    def __ne__(self, other):  # pragma: no cover
        return not self.__eq__(other)

    @classmethod
    def from_json(cls, json_string):
        return cls(**json.loads(json_string))

    @classmethod
    def from_dict(cls, json_dict):
        return cls(**json_dict)

    def get_json_data(self):
        """
        Get the Dict of the properties

        :return: dict of properties
        """
        result = {}
        for name, prop in self._properties.items():
            item = getattr(self, name, None)
            if item is not None:
                if isinstance(item, BaseAbstractEntity):
                    item = item.get_json_data()
                elif isinstance(item, array_types):
                    tmp = []
                    for i in item:
                        if isinstance(i, BaseAbstractEntity):
                            tmp.append(i.get_json_data())
                        else:
                            tmp.append(i)
                    item = tmp
                elif isinstance(item, dict):
                    tmp = {}
                    for k, v in item.items():
                        if isinstance(v, BaseAbstractEntity):
                            tmp[k] = v.get_json_data()
                        else:
                            tmp[k] = v
                    item = tmp
            if item is not None or prop.include_if_null:
                result[prop.json_name] = item
        return result

    def to_json(self):
        """

        :return: json string of the data
        """
        return json.dumps(self.get_json_data())

    def __getitem__(self, item):
        value = self._properties.get(item, None)
        if value is not None:
            return getattr(self, item)
        raise AttributeError(item)

    def __setitem__(self, key, value):
        prop = self._properties.get(key)
        if prop is not None:
            setattr(key, value)

    def __delitem__(self, key):
        prop = self._properties.get(key)
        if prop is not None:
            delattr(self, key)
        else:
            raise AttributeError(key)

    def __contains__(self, item):
        return item in self._properties.keys()

    def __len__(self):
        return len(self._properties)

    def __iter__(self):
        for item in self._properties.keys():
            yield item

    def items(self):
        items = []
        for key in self._properties.keys():
            items.append((key, getattr(self, key)))
        return items

    def keys(self):
        return self._properties.keys()

    def values(self):
        items = []
        for key in self._properties.keys():
            items.append(getattr(self, key))
        return items


class BaseEntityMetaType(type):
    def __new__(mcs, name, bases, body):
        prop_dict = OrderedDict()

        for base in bases:
            for k, v in getattr(base, '_properties', {}).items():
                prop_dict.setdefault(k, v)

        def _transform_property(prop_name, prop_obj):
            prop_dict[prop_name] = prop_obj
            prop_obj.set_property_name(prop_name)
            _get = lambda self: self._values[prop_name].getval()
            _set = lambda self, val: self._values[prop_name].setval(val)
            _del = lambda self: self._values[prop_name].delval()
            body[prop_name] = property(_get, _set, _del)

        property_definitions = [(k, v) for k, v in body.items() if isinstance(v, JsonProperty)]

        for k, v in property_definitions:
            _transform_property(k, v)

        json_names = set()
        for v in prop_dict.values():
            # type: v -> EntityProperty
            if v.json_name in json_names:
                raise EntityModelException("%s defines the json property %s more than once" % (name, v.json_name))
            json_names.add(v.json_name)

        body['_properties'] = prop_dict

        # Create the class
        klass = super(BaseEntityMetaType, mcs).__new__(mcs, name, bases, body)

        return klass


def collection_to_json(collection):
    result = []
    for item in collection:
        if isinstance(item, BaseAbstractEntity):
            result.append(item.get_json_data())
        elif isinstance(item, (list, tuple)):
            sub_result = []
            for sub_item in item:
                if isinstance(sub_item, BaseAbstractEntity):
                    sub_result.append(sub_item.get_json_data())
            result.append(sub_result)
        elif isinstance(item, dict):
            sub_result = {}
            for k, v in item:
                if isinstance(v, BaseAbstractEntity):
                    sub_result[k] = v.get_json_data()
            result.append(sub_result)

    return json.dumps(result)


@add_metaclass(BaseEntityMetaType)
class BaseEntity(BaseAbstractEntity):
    _id = None

    def __init__(self, **values):
        self._id = values.pop('id', None)
        super(BaseEntity, self).__init__(**values)

    @property
    def id(self):
        """ Get the Id of the entity

        :return: id of entity
        """
        return self._id

    def __eq__(self, other):
        if not isinstance(other, BaseAbstractEntity):  # pragma: no cover
            return False

        return self._id == other.id

    def __ne__(self, other):  # pragma: no cover
        return not self.__eq__(other)

    def get_json_data(self):
        """
        Get the Dict of the properties

        :return: dict of properties
        """
        result = {}
        for name, prop in self._properties.items():
            item = getattr(self, name, None)
            if item is not None:
                if isinstance(item, BaseAbstractEntity):
                    item = item.get_json_data()
                elif item is not None and isinstance(item, array_types):
                    tmp = []
                    for i in item:
                        if isinstance(i, BaseAbstractEntity):
                            tmp.append(i.get_json_data())
                        else:
                            tmp.append(i)
                    item = tmp
                elif item is not None and isinstance(item, dict):
                    tmp = {}
                    for k, v in item.items():
                        if isinstance(v, BaseAbstractEntity):
                            tmp[k] = v.get_json_data()
                        else:
                            tmp[k] = v
                    item = tmp
            if item is not None or prop.include_if_null:
                result[prop.json_name] = item
        if self.id is not None:
            result['id'] = self.id
        return result


__all__ = ['BaseEntity', 'BaseAbstractEntity', 'BaseEntityMetaType', 'collection_to_json', 'EntityModelException',
           'JsonProperty']



from __future__ import unicode_literals

import json
import requests
from requests import exceptions as rexc

from wiremock._compat import integer_types, float_types, array_types, string_types
from wiremock.base.base_entity import BaseAbstractEntity
from wiremock.constants import make_headers, Config, logger
from wiremock.exceptions import *


class RestClient(object):

    def __init__(self, timeout=None, base_url=None):
        self.timeout = timeout
        self.base_url = base_url

    def _base_url(self):
        return self.base_url or Config.base_url

    def _timeout(self):
        return self.timeout or Config.timeout

    def _log(self, action, url, **kwargs):
        ctx = {'timeout': kwargs.get('timeout')}
        logger.debug(
            "%s [%s] - %s", action, url, kwargs.get('json', json.dumps(kwargs.get('data', None))),
            extra=ctx
        )

    def post(self, uri, **kwargs):
        if 'timeout' not in kwargs:
            kwargs['timeout'] = self._timeout()
        try:
            url = self._base_url() + uri
            self._log("POST", url, **kwargs)
            return requests.post(url, **kwargs)
        except rexc.Timeout as e:  # pragma: no cover
            raise TimeoutException(-1, e)
        except rexc.ConnectionError as e:  # pragma: no cover
            raise ApiUnavailableException(-1, e)

    def get(self, uri, **kwargs):
        if 'timeout' not in kwargs:
            kwargs['timeout'] = self._timeout()
        try:
            url = self._base_url() + uri
            self._log("GET", url, **kwargs)
            return requests.get(url, **kwargs)
        except rexc.Timeout as e:  # pragma: no cover
            raise TimeoutException(-1, e)
        except rexc.ConnectionError as e:  # pragma: no cover
            raise ApiUnavailableException(-1, e)

    def put(self, uri, **kwargs):
        if 'timeout' not in kwargs:
            kwargs['timeout'] = self._timeout()
        try:
            url = self._base_url() + uri
            self._log("PUT", url, **kwargs)
            return requests.put(url, **kwargs)
        except rexc.Timeout as e:  # pragma: no cover
            raise TimeoutException(-1, e)
        except rexc.ConnectionError as e:  # pragma: no cover
            raise ApiUnavailableException(-1, e)

    def patch(self, uri, **kwargs):  # pragma: no cover
        if 'timeout' not in kwargs:
            kwargs['timeout'] = self._timeout()
        try:
            url = self._base_url() + uri
            self._log("PATCH", url, **kwargs)
            return requests.patch(url, **kwargs)
        except rexc.Timeout as e:  # pragma: no cover
            raise TimeoutException(-1, e)
        except rexc.ConnectionError as e:  # pragma: no cover
            raise ApiUnavailableException(-1, e)

    def delete(self, uri, **kwargs):
        if 'timeout' not in kwargs:
            kwargs['timeout'] = self._timeout()
        try:
            url = self._base_url() + uri
            self._log("DELETE", url, **kwargs)
            return requests.delete(url, **kwargs)
        except rexc.Timeout as e:  # pragma: no cover
            raise TimeoutException(-1, e)
        except rexc.ConnectionError as e:  # pragma: no cover
            raise ApiUnavailableException(-1, e)

    def options(self, uri, **kwargs):  # pragma: no cover
        if 'timeout' not in kwargs:
            kwargs['timeout'] = self._timeout()
        try:
            url = self._base_url() + uri
            self._log("OPTIONS", url, **kwargs)
            return requests.options(url, **kwargs)
        except rexc.Timeout as e:  # pragma: no cover
            raise TimeoutException(-1, e)
        except rexc.ConnectionError as e:  # pragma: no cover
            raise ApiUnavailableException(-1, e)

    def head(self, uri, **kwargs):  # pragma: no cover
        if 'timeout' not in kwargs:
            kwargs['timeout'] = self._timeout()
        try:
            url = self._base_url() + uri
            self._log("HEAD", url, **kwargs)
            return requests.head(url, **kwargs)
        except rexc.Timeout as e:  # pragma: no cover
            raise TimeoutException(-1, e)
        except rexc.ConnectionError as e:  # pragma: no cover
            raise ApiUnavailableException(-1, e)

    @staticmethod
    def handle_response(response):
        sc = response.status_code
        if sc in [200, 201, 204]:
            return response
        elif sc is 401:  # pragma: no cover
            raise RequiresLoginException(sc, response.text)
        elif sc is 403:  # pragma: no cover
            raise ForbiddenException(sc, response.text)
        elif sc is 404:  # pragma: no cover
            raise NotFoundException(sc, response.text)
        elif sc is 422:  # pragma: no cover
            raise InvalidInputException(sc, response.text)
        elif 200 < sc < 400:  # pragma: no cover
            raise UnexpectedResponseException(sc, response.text)
        elif 400 <= sc < 500 and sc is not 404:  # pragma: no cover
            raise ClientException(sc, response.text)
        elif 500 <= sc < 600:  # pragma: no cover
            raise ServerException(sc, response.text)
        else:  # pragma: no cover
            raise ApiException(sc, response.text)


class BaseResource(object):

    REST_CLIENT = RestClient()

    @classmethod
    def endpoint(cls):
        return '/'  # pragma: no cover

    @classmethod
    def endpoint_single(cls):
        return '/{id}'  # pragma: no cover

    @classmethod
    def entity_class(cls):
        return None  # pragma: no cover

    @classmethod
    def get_base_uri(cls, endpoint, **id_dict):
        if id_dict:
            return endpoint.format(**id_dict)
        return endpoint

    @staticmethod
    def get_entity_id(entity_id, entityClass):
        if not (isinstance(entity_id, integer_types) or isinstance(entity_id, string_types)
                or isinstance(entity_id, entityClass)):

            raise InvalidInputException(422, entity_id)
        if isinstance(entity_id, entityClass):
            entity_id = entity_id.id

        return entity_id

    @staticmethod
    def validate_is_entity(entity, entityClass):
        if not isinstance(entity, entityClass):
            raise InvalidInputException(422, entity)

    @classmethod
    def _create(cls, entity, parameters=None, ids={}):  # pragma: no cover
        if isinstance(entity, BaseAbstractEntity):
            response = cls.REST_CLIENT.post(
                cls.get_base_uri(cls.endpoint(), **ids), json=entity.get_json_data(), headers=make_headers(),
                params=parameters
            )
        else:
            response = cls.REST_CLIENT.post(
                cls.get_base_uri(cls.endpoint(), **ids), data=json.dumps(entity), headers=make_headers(),
                params=parameters
            )

        response = cls.REST_CLIENT.handle_response(response)

        if cls.entity_class() is None or not issubclass(cls.entity_class(), BaseAbstractEntity):
            return response  # pragma: no cover
        else:
            return cls.entity_class().from_dict(response.json())

    @classmethod
    def _update(cls, entity, parameters=None, ids={}):  # pragma: no cover
        entity_id = getattr(entity, 'id', None)
        if entity_id is not None:
            ids['id'] = entity_id
        if isinstance(entity, BaseAbstractEntity):
            response = cls.REST_CLIENT.put(
                cls.get_base_uri(cls.endpoint_single(), **ids), json=entity.get_json_data(), headers=make_headers(),
                params=parameters
            )
        else:
            response = cls.REST_CLIENT.put(
                cls.get_base_uri(cls.endpoint_single(), **ids), data=json.dumps(entity), headers=make_headers(),
                params=parameters
            )

        response = cls.REST_CLIENT.handle_response(response)

        if cls.entity_class() is None or not issubclass(cls.entity_class(), BaseAbstractEntity):
            return response  # pragma: no cover
        else:
            return cls.entity_class().from_dict(response.json())

    @classmethod
    def _partial_update(cls, entity, parameters=None, ids={}):  # pragma: no cover
        entity_id = getattr(entity, 'id', None)
        if entity_id is not None:
            ids['id'] = entity_id
        if isinstance(entity, BaseAbstractEntity):
            response = cls.REST_CLIENT.patch(
                cls.get_base_uri(cls.endpoint_single(), **ids), json=entity.get_json_data(), headers=make_headers(),
                params=parameters
            )
        else:
            response = cls.REST_CLIENT.patch(
                cls.get_base_uri(cls.endpoint_single(), **ids), data=json.dumps(entity), headers=make_headers(),
                params=parameters
            )

        response = cls.REST_CLIENT.handle_response(response)

        if cls.entity_class() is None or not issubclass(cls.entity_class(), BaseAbstractEntity):
            return response  # pragma: no cover
        else:
            return cls.entity_class().from_dict(response.json())

    @classmethod
    def _retreive_all(cls, parameters=None, ids={}):  # pragma: no cover
        response = cls.REST_CLIENT.get(
            cls.get_base_uri(cls.endpoint(), **ids), headers=make_headers(), params=parameters
        )
        response = cls.REST_CLIENT.handle_response(response)

        if cls.entity_class() is None or not issubclass(cls.entity_class(), BaseAbstractEntity):
            return response  # pragma: no cover
        else:
            response_json = response.json()
            if isinstance(response_json, array_types):
                results = []
                for r in response_json:
                    if isinstance(r, dict):
                        results.append(cls.entity_class().from_dict(r))
                return results
            else:
                return cls.entity_class().from_dict(response.json())

    @classmethod
    def _retreive_one(cls, entity, parameters=None, ids={}):  # pragma: no cover
        if isinstance(entity, integer_types) or isinstance(entity, float_types):
            ids['id'] = entity
            response = cls.REST_CLIENT.get(
                cls.get_base_uri(cls.endpoint_single(), **ids), headers=make_headers(), params=parameters
            )
        elif entity is not None and issubclass(entity, BaseAbstractEntity):
            entity_id = getattr(entity, 'id', None)
            ids['id'] = entity_id
            response = cls.REST_CLIENT.get(
                cls.get_base_uri(cls.endpoint_single(), **ids), headers=make_headers(), params=parameters
            )
        else:
            response = cls.REST_CLIENT.get(
                cls.get_base_uri(cls.endpoint_single(), **ids), headers=make_headers(), params=parameters
            )

        response = cls.REST_CLIENT.handle_response(response)

        if cls.entity_class() is None or not issubclass(cls.entity_class(), BaseAbstractEntity):
            return response  # pragma: no cover
        else:
            return cls.entity_class().from_dict(response.json())

    @classmethod
    def _delete(cls, entity, parameters=None, ids={}):  # pragma: no cover
        if isinstance(entity, integer_types) or isinstance(entity, float_types):
            ids['id'] = entity
            response = cls.REST_CLIENT.delete(
                cls.get_base_uri(cls.endpoint_single(), **ids), headers=make_headers(), params=parameters
            )
        elif isinstance(entity, BaseAbstractEntity):
            entity_id = getattr(entity, 'id', None)
            ids['id'] = entity_id
            response = cls.REST_CLIENT.delete(
                cls.get_base_uri(cls.endpoint_single(), **ids), headers=make_headers(), params=parameters
            )
        else:
            response = cls.REST_CLIENT.delete(
                cls.get_base_uri(cls.endpoint_single(), **ids), headers=make_headers(), params=parameters
            )

        response = cls.REST_CLIENT.handle_response(response)
        if response is None:
            return entity

        if cls.entity_class() is None or not issubclass(cls.entity_class(), BaseAbstractEntity):
            return response  # pragma: no cover
        else:
            try:
                return cls.entity_class().from_dict(response.json())
            except ValueError:
                return response  # pragma: no cover


__all__ = ['RestClient', 'BaseResource']


from __future__ import unicode_literals

from wiremock.base.base_resource import BaseResource
from wiremock.resources.settings import GlobalSetting


class GlobalSettings(BaseResource):

    @classmethod
    def endpoint(cls):
        return '/settings'

    @classmethod
    def endpoint_single(cls):
        return '/settings'

    @classmethod
    def entity_class(cls):
        return GlobalSetting

    @classmethod
    def update_global_settings(cls, settings, parameters={}):
        return cls._create(settings, parameters=parameters)


__all__ = ['GlobalSettings']



from __future__ import unicode_literals

from wiremock._compat import add_metaclass
from wiremock.base import BaseEntity, JsonProperty, BaseAbstractEntity, BaseEntityMetaType


class HttpMethods(object):
    ANY = 'ANY'
    GET = 'GET'
    POST = 'POST'
    PUT = 'PUT'
    PATCH = 'PATCH'
    DELETE = 'DELETE'
    OPTIONS = 'OPTIONS'
    HEAD = 'HEAD'


class CommonHeaders(object):
    ACCESS_CONTROL_ALLOW_ORIGIN = 'Access-Control-Allow-Origin'
    ACCEPT = 'Accept'
    ACCEPT_CHARSET = 'Accept-Charset'
    ACCEPT_ENCODING = 'Accept-Encoding'
    ACCEPT_LANGUAGE = 'Accept-Language'
    ACCEPT_DATETIME = 'Accept-Datetime'
    ACCEPT_PATCH = 'Accept-Patch'
    ACCEPT_RANGES = 'Accept-Ranges'
    AGE = 'Age'
    ALLOW = 'Allow'
    AUTHORIZATION = 'Authorization'
    CACHE_CONTROL = 'Cache-Control'
    CONNECTION = 'Connection'
    COOKIE = 'Cookie'
    CONTENT_LENGTH = 'Content-Length'
    CONTENT_MD5 = 'Content-MD5'
    CONTENT_TYPE = 'Content-Type'
    DATE = 'Date'
    DNT = 'DNT'
    ETAG = 'ETag'
    EXPECT = 'Expect'
    EXPIRES = 'Expires'
    FORWARDED = 'Forwarded'
    FROM = 'From'
    HOST = 'Host'
    IF_MATCH = 'If-Match'
    IF_MODIFIED_SINCE = 'If-Modified-Since'
    IF_NONE_MATCH = 'If-None-Match'
    IF_RANGE = 'If-Range'
    IF_UNMODIFIED_SINCE = 'If-Unmodified-Since'
    LAST_MODIFIED = 'Last-Modified'
    LINK = 'Link'
    LOCATION = 'Location'
    MAX_FORWARDS = 'Max-Forwards'
    ORIGIN = 'Origin'
    PRAGMA = 'Pragma'
    PROXY_AUTHORIZATION = 'Proxy-Authorization'
    PROXY_CONNECTION = 'Proxy-Connection'
    RANGE = 'Range'
    REFERER = 'Referer'
    SERVER = 'Server'
    SET_COOKIE = 'Set-Cookie'
    TE = 'TE'
    USER_AGENT = 'user-agent'
    UPGRADE = 'Upgrade'
    VIA = 'Via'
    WARNING = 'Warning'
    X_CSRF_TOKEN = 'X-Csrf-TOken'
    X_FORWARDED_FOR = 'X-Forwarded-For'
    X_FORWARDED_HOST = 'X-Forwarded-Host'
    X_FORWARDED_PROTO = 'X-Forwarded-Proto'
    X_REQUEST_ID = 'X-Request-ID'
    X_CORRELATION_ID = 'X-Correlation-ID'
    X_REQUESTED_WITH = 'X-Requested-With'


class WireMockMatchers(object):
    ABSENT = 'absent'
    ANYTHING = 'anything'
    CASE_INSENSITIVE = 'caseInsensitive'  # Should be true/false
    CONTAINS = 'contains'
    DOES_NOT_MATCH = 'doesNotMatch'
    EQUAL_TO = 'equalTo'
    EQUAL_TO_JSON = 'equalToJson'
    EQUAL_TO_XML = 'equalToXml'
    IGNORE_ARRAY_ORDER = 'ignoreArrayOrder'  # Should be true/false
    IGNORE_EXTRA_ELEMENTS = 'ignoreExtraElements'  # Should be true/false
    MATCHES = 'matches'
    MATCHES_JSON_PATH = 'matchesJsonPath'
    MATCHES_X_PATH = 'matchesXPath'
    X_PATH_NAMESPACES = 'xPathNamespaces'


@add_metaclass(BaseEntityMetaType)
class BasicAuthCredentials(BaseAbstractEntity):
    username = JsonProperty('username')
    password = JsonProperty('password')


class DelayDistributionMethods(object):
    LOG_NORMAL = 'lognormal'
    UNIFORM = 'uniform'


class ResponseFaultType(object):
    EMPTY_RESPONSE = 'EMPTY_RESPONSE'
    MALFORMED_RESPONSE_CHUNK = 'MALFORMED_RESPONSE_CHUNK'
    RANDOM_DATA_THEN_CLOSE = 'RANDOM_DATA_THEN_CLOSE'


@add_metaclass(BaseEntityMetaType)
class DelayDistribution(BaseAbstractEntity):
    distribution_type = JsonProperty('type')

    # lognormal
    median = JsonProperty('median')
    sigma = JsonProperty('sigma')

    # uniform
    upper = JsonProperty('upper')
    lower = JsonProperty('lower')


@add_metaclass(BaseEntityMetaType)
class MappingRequest(BaseAbstractEntity):
    method = JsonProperty('method')
    url = JsonProperty('url')
    url_path = JsonProperty('urlPath')
    url_path_pattern = JsonProperty('urlPathPattern')
    url_pattern = JsonProperty('urlPattern')
    basic_auth_credentials = JsonProperty('basicAuthCredentials', klass=BasicAuthCredentials)
    cookies = JsonProperty('cookies', klass=dict)
    headers = JsonProperty('headers', klass=dict)
    query_parameters = JsonProperty('queryParameters', klass=dict)
    body_patterns = JsonProperty('bodyPatterns', klass=dict)


@add_metaclass(BaseEntityMetaType)
class MappingResponse(BaseAbstractEntity):
    additional_proxy_request_headers = JsonProperty('additionalProxyRequestHeaders', klass=dict)
    base64_body = JsonProperty('base64Body')
    body = JsonProperty('body')
    body_file_name = JsonProperty('bodyFileName')
    json_body = JsonProperty('jsonBody')
    delay_distribution = JsonProperty('delayDistribution', klass=DelayDistribution)
    fault = JsonProperty('fault')
    fixed_delay_milliseconds = JsonProperty('fixedDelayMilliseconds')
    from_configured_stub = JsonProperty('fromConfiguredStub')
    headers = JsonProperty('headers', klass=dict)
    proxy_base_url = JsonProperty('proxyBaseUrl')
    status = JsonProperty('status')
    status_message = JsonProperty('statusMessage')
    transformer_parameters = JsonProperty('transformerParameters', klass=dict)
    transformers = JsonProperty('transformers', klass=list)


class Mapping(BaseEntity):
    priority = JsonProperty('priority')
    request = JsonProperty('request', klass=MappingRequest)
    response = JsonProperty('response', klass=MappingResponse)
    persistent = JsonProperty('persistent')
    post_serve_actions = JsonProperty('postServeActions', klass=dict)
    new_scenario_state = JsonProperty('newScenarioState')
    required_scenario_state = JsonProperty('requiredScenarioState')
    scenario_name = JsonProperty('scenarioName')


@add_metaclass(BaseEntityMetaType)
class MappingMeta(BaseAbstractEntity):
    total = JsonProperty('total')


@add_metaclass(BaseEntityMetaType)
class AllMappings(BaseAbstractEntity):
    mappings = JsonProperty('mappings', klass=list, list_klass=Mapping)
    meta = JsonProperty('meta', klass=MappingMeta)


__all__ = ['Mapping', 'MappingResponse', 'MappingRequest', 'DelayDistribution', 'ResponseFaultType',
           'DelayDistributionMethods', 'BasicAuthCredentials', 'WireMockMatchers', 'HttpMethods',
           'CommonHeaders', 'MappingMeta', 'AllMappings']



from __future__ import unicode_literals

from wiremock.tests.base import BaseClientTestCase, attr
from wiremock.resources.mappings import HttpMethods, CommonHeaders, BasicAuthCredentials
from wiremock.resources.near_misses import NearMissMatchPatternRequest, NearMissMatchResponse, NearMissMatchRequest, \
    NearMissMatch, NearMissMatchResult, NearMissRequestPatternResult


class NearMissesSerializationTests(BaseClientTestCase):

    @attr('unit', 'serialization', 'nearmisses')
    def test_near_miss_match_pattern_request_serialization(self):
        e = NearMissMatchPatternRequest(
            url='test', url_pattern='test2', url_path='test3', url_path_pattern='test4',
            method=HttpMethods.GET, client_ip='1.1.1.1', headers={CommonHeaders.ACCEPT: 'json'},
            query_parameters={'test': 1}, cookies={'chocolate': 'chip'}, body_patterns={'test': 3},
            basic_auth_credentials=BasicAuthCredentials(username='username', password='password'),
            browser_proxy_request=False, logged_date=12345, logged_date_string='1/1/2017 00:00:00+0000'
        )
        serialized = e.get_json_data()
        self.assertDictContainsKeyWithValue(serialized, 'url', 'test')
        self.assertDictContainsKeyWithValue(serialized, 'urlPattern', 'test2')
        self.assertDictContainsKeyWithValue(serialized, 'urlPath', 'test3')
        self.assertDictContainsKeyWithValue(serialized, 'urlPathPattern', 'test4')
        self.assertDictContainsKeyWithValue(serialized, 'method', 'GET')
        self.assertDictContainsKeyWithValue(serialized, 'clientIp', '1.1.1.1')
        self.assertDictContainsKeyWithValue(serialized, 'headers', {'Accept': 'json'})
        self.assertDictContainsKeyWithValue(serialized, 'queryParameters', {'test': 1})
        self.assertDictContainsKeyWithValue(serialized, 'cookies', {'chocolate': 'chip'})
        self.assertDictContainsKeyWithValue(serialized, 'bodyPatterns', {'test': 3})
        self.assertDictContainsKeyWithValue(serialized, 'basicAuthCredentials', {'username': 'username', 'password': 'password'})
        self.assertDictContainsKeyWithValue(serialized, 'browserProxyRequest', False)
        self.assertDictContainsKeyWithValue(serialized, 'loggedDate', 12345)
        self.assertDictContainsKeyWithValue(serialized, 'loggedDateString', '1/1/2017 00:00:00+0000')

    @attr('unit', 'serialization', 'nearmisses')
    def test_near_miss_match_pattern_request_deserialization(self):
        serialized = {'clientIp': '1.1.1.1', 'cookies': {'chocolate': 'chip'}, 'loggedDate': 12345,
                      'urlPattern': 'test2', 'headers': {'Accept': 'json'}, 'url': 'test', 'urlPath': 'test3',
                      'urlPathPattern': 'test4', 'browserProxyRequest': False,
                      'loggedDateString': '1/1/2017 00:00:00+0000', 'bodyPatterns': {'test': 3},
                      'queryParameters': {'test': 1},
                      'basicAuthCredentials': {'username': 'username', 'password': 'password'}, 'method': 'GET'
                      }
        e = NearMissMatchPatternRequest.from_dict(serialized)
        self.assertIsInstance(e, NearMissMatchPatternRequest)
        self.assertEquals('test', e.url)
        self.assertEquals('test2', e.url_pattern)
        self.assertEquals('test3', e.url_path)
        self.assertEquals('test4', e.url_path_pattern)
        self.assertEquals('GET', e.method)
        self.assertEquals('1.1.1.1', e.client_ip)
        self.assertEquals({'Accept': 'json'}, e.headers)
        self.assertEquals({'test': 1}, e.query_parameters)
        self.assertEquals({'chocolate': 'chip'}, e.cookies)
        self.assertDictEqual({'test': 3}, e.body_patterns)
        self.assertIsInstance(e.basic_auth_credentials, BasicAuthCredentials)
        self.assertEquals('username', e.basic_auth_credentials.username)
        self.assertEquals('password', e.basic_auth_credentials.password)
        self.assertEquals(False, e.browser_proxy_request)
        self.assertEquals(12345, e.logged_date)
        self.assertEquals('1/1/2017 00:00:00+0000', e.logged_date_string)

    @attr('unit', 'serialization', 'nearmisses')
    def test_near_miss_match_request_serialization(self):
        e = NearMissMatchRequest(
            url='test', absolute_url='test2', method=HttpMethods.GET, client_ip='1.1.1.1',
            headers={CommonHeaders.ACCEPT: 'json'}, query_parameters={'test': 1},
            cookies={'chocolate': 'chip'},
            basic_auth_credentials=BasicAuthCredentials(username='username', password='password'),
            browser_proxy_request=False, logged_date=12345, logged_date_string='1/1/2017 00:00:00+0000',
            body_as_base64='test3', body='test4'
        )
        serialized = e.get_json_data()
        self.assertDictContainsKeyWithValue(serialized, 'url', 'test')
        self.assertDictContainsKeyWithValue(serialized, 'absoluteUrl', 'test2')
        self.assertDictContainsKeyWithValue(serialized, 'method', 'GET')
        self.assertDictContainsKeyWithValue(serialized, 'clientIp', '1.1.1.1')
        self.assertDictContainsKeyWithValue(serialized, 'headers', {'Accept': 'json'})
        self.assertDictContainsKeyWithValue(serialized, 'queryParameters', {'test': 1})
        self.assertDictContainsKeyWithValue(serialized, 'cookies', {'chocolate': 'chip'})
        self.assertDictContainsKeyWithValue(serialized, 'basicAuthCredentials', {'username': 'username', 'password': 'password'})
        self.assertDictContainsKeyWithValue(serialized, 'browserProxyRequest', False)
        self.assertDictContainsKeyWithValue(serialized, 'loggedDate', 12345)
        self.assertDictContainsKeyWithValue(serialized, 'loggedDateString', '1/1/2017 00:00:00+0000')
        self.assertDictContainsKeyWithValue(serialized, 'bodyAsBase64', 'test3')
        self.assertDictContainsKeyWithValue(serialized, 'body', 'test4')

    @attr('unit', 'serialization', 'nearmisses')
    def test_near_miss_match_request_deserialization(self):
        serialized = {'clientIp': '1.1.1.1', 'cookies': {'chocolate': 'chip'}, 'loggedDate': 12345,
                      'absoluteUrl': 'test2', 'headers': {'Accept': 'json'}, 'url': 'test',
                      'browserProxyRequest': False, 'body': 'test4', 'bodyAsBase64': 'test3',
                      'loggedDateString': '1/1/2017 00:00:00+0000', 'queryParameters': {'test': 1},
                      'basicAuthCredentials': {'username': 'username', 'password': 'password'}, 'method': 'GET'
                      }
        e = NearMissMatchRequest.from_dict(serialized)
        self.assertIsInstance(e, NearMissMatchRequest)
        self.assertEquals('test', e.url)
        self.assertEquals('test2', e.absolute_url)
        self.assertEquals('GET', e.method)
        self.assertEquals('1.1.1.1', e.client_ip)
        self.assertEquals({'Accept': 'json'}, e.headers)
        self.assertEquals({'test': 1}, e.query_parameters)
        self.assertEquals({'chocolate': 'chip'}, e.cookies)
        self.assertIsInstance(e.basic_auth_credentials, BasicAuthCredentials)
        self.assertEquals('username', e.basic_auth_credentials.username)
        self.assertEquals('password', e.basic_auth_credentials.password)
        self.assertEquals(False, e.browser_proxy_request)
        self.assertEquals(12345, e.logged_date)
        self.assertEquals('1/1/2017 00:00:00+0000', e.logged_date_string)
        self.assertEquals('test3', e.body_as_base64)
        self.assertEquals('test4', e.body)

    @attr('unit', 'serialization', 'nearmisses')
    def test_near_miss_match_result_serialization(self):
        e = NearMissMatchResult(distance=0.75)
        serialized = e.get_json_data()
        self.assertDictContainsKeyWithValue(serialized, 'distance', 0.75)

    @attr('unit', 'serialization', 'nearmisses')
    def test_near_miss_match_result_deserialization(self):
        serialized = {'distance': 0.75}
        e = NearMissMatchResult.from_dict(serialized)
        self.assertIsInstance(e, NearMissMatchResult)
        self.assertEquals(0.75, e.distance)

    @attr('unit', 'serialization', 'nearmisses')
    def test_near_miss_request_pattern_result_serialization(self):
        e = NearMissRequestPatternResult(
            url='test', absolute_url='test2', method=HttpMethods.GET, client_ip='1.1.1.1',
            headers={CommonHeaders.ACCEPT: 'json'}, query_parameters={'test': 1},
            cookies={'chocolate': 'chip'},
            basic_auth_credentials=BasicAuthCredentials(username='username', password='password'),
            browser_proxy_request=False, body_as_base64='test3', body='test4'
        )
        serialized = e.get_json_data()
        self.assertDictContainsKeyWithValue(serialized, 'url', 'test')
        self.assertDictContainsKeyWithValue(serialized, 'absoluteUrl', 'test2')
        self.assertDictContainsKeyWithValue(serialized, 'method', 'GET')
        self.assertDictContainsKeyWithValue(serialized, 'clientIp', '1.1.1.1')
        self.assertDictContainsKeyWithValue(serialized, 'headers', {'Accept': 'json'})
        self.assertDictContainsKeyWithValue(serialized, 'queryParameters', {'test': 1})
        self.assertDictContainsKeyWithValue(serialized, 'cookies', {'chocolate': 'chip'})
        self.assertDictContainsKeyWithValue(serialized, 'basicAuthCredentials', {'username': 'username', 'password': 'password'})
        self.assertDictContainsKeyWithValue(serialized, 'browserProxyRequest', False)
        self.assertDictContainsKeyWithValue(serialized, 'bodyAsBase64', 'test3')
        self.assertDictContainsKeyWithValue(serialized, 'body', 'test4')

    @attr('unit', 'serialization', 'nearmisses')
    def test_near_miss_request_pattern_result_deserialization(self):
        serialized = {'clientIp': '1.1.1.1', 'cookies': {'chocolate': 'chip'}, 'absoluteUrl': 'test2',
                      'headers': {'Accept': 'json'}, 'url': 'test', 'browserProxyRequest': False,
                      'body': 'test4', 'bodyAsBase64': 'test3', 'queryParameters': {'test': 1},
                      'basicAuthCredentials': {'username': 'username', 'password': 'password'}, 'method': 'GET'
                      }
        e = NearMissRequestPatternResult.from_dict(serialized)
        self.assertIsInstance(e, NearMissRequestPatternResult)
        self.assertEquals('test', e.url)
        self.assertEquals('test2', e.absolute_url)
        self.assertEquals('GET', e.method)
        self.assertEquals('1.1.1.1', e.client_ip)
        self.assertEquals({'Accept': 'json'}, e.headers)
        self.assertEquals({'test': 1}, e.query_parameters)
        self.assertEquals({'chocolate': 'chip'}, e.cookies)
        self.assertIsInstance(e.basic_auth_credentials, BasicAuthCredentials)
        self.assertEquals('username', e.basic_auth_credentials.username)
        self.assertEquals('password', e.basic_auth_credentials.password)
        self.assertEquals(False, e.browser_proxy_request)
        self.assertEquals('test3', e.body_as_base64)
        self.assertEquals('test4', e.body)

    @attr('unit', 'serialization', 'nearmisses')
    def test_near_miss_match_serialization(self):
        e = NearMissMatch(
            request=NearMissMatchRequest(url='test'),
            request_pattern=NearMissRequestPatternResult(url='test2'),
            match_result=NearMissMatchResult(distance=0.75)
        )
        serialized = e.get_json_data()
        self.assertDictContainsKeyWithValueType(serialized, 'request', dict)
        request = serialized['request']
        self.assertDictContainsKeyWithValue(request, 'url', 'test')

        self.assertDictContainsKeyWithValueType(serialized, 'requestPattern', dict)
        request_pattern = serialized['requestPattern']
        self.assertDictContainsKeyWithValue(request_pattern, 'url', 'test2')

        self.assertDictContainsKeyWithValueType(serialized, 'matchResult', dict)
        match_result = serialized['matchResult']
        self.assertDictContainsKeyWithValue(match_result, 'distance', 0.75)

    @attr('unit', 'serialization', 'nearmisses')
    def test_near_miss_match_deserialization(self):
        serialized = {
            'request': {
                'clientIp': '1.1.1.1', 'cookies': {'chocolate': 'chip'}, 'loggedDate': 12345,
                'absoluteUrl': 'test2', 'headers': {'Accept': 'json'}, 'url': 'test',
                'browserProxyRequest': False, 'body': 'test4', 'bodyAsBase64': 'test3',
                'loggedDateString': '1/1/2017 00:00:00+0000', 'queryParameters': {'test': 1},
                'basicAuthCredentials': {'username': 'username', 'password': 'password'}, 'method': 'GET'
            },
            'requestPattern': {
                'clientIp': '1.1.1.1', 'cookies': {'chocolate': 'chip'}, 'absoluteUrl': 'test2',
                'headers': {'Accept': 'json'}, 'url': 'test', 'browserProxyRequest': False,
                'body': 'test4', 'bodyAsBase64': 'test3', 'queryParameters': {'test': 1},
                'basicAuthCredentials': {'username': 'username', 'password': 'password'}, 'method': 'GET'
            },
            'matchResult': {'distance': 0.75}
        }
        e = NearMissMatch.from_dict(serialized)
        self.assertIsInstance(e, NearMissMatch)
        self.assertIsInstance(e.request, NearMissMatchRequest)
        self.assertIsInstance(e.request_pattern, NearMissRequestPatternResult)
        self.assertIsInstance(e.match_result, NearMissMatchResult)

        # request
        self.assertEquals('test', e.request.url)
        self.assertEquals('test2', e.request.absolute_url)
        self.assertEquals('GET', e.request.method)
        self.assertEquals('1.1.1.1', e.request.client_ip)
        self.assertEquals({'Accept': 'json'}, e.request.headers)
        self.assertEquals({'test': 1}, e.request.query_parameters)
        self.assertEquals({'chocolate': 'chip'}, e.request.cookies)
        self.assertIsInstance(e.request.basic_auth_credentials, BasicAuthCredentials)
        self.assertEquals('username', e.request.basic_auth_credentials.username)
        self.assertEquals('password', e.request.basic_auth_credentials.password)
        self.assertEquals(False, e.request.browser_proxy_request)
        self.assertEquals(12345, e.request.logged_date)
        self.assertEquals('1/1/2017 00:00:00+0000', e.request.logged_date_string)
        self.assertEquals('test3', e.request.body_as_base64)
        self.assertEquals('test4', e.request.body)

        # request pattern
        self.assertEquals('test', e.request_pattern.url)
        self.assertEquals('test2', e.request_pattern.absolute_url)
        self.assertEquals('GET', e.request_pattern.method)
        self.assertEquals('1.1.1.1', e.request_pattern.client_ip)
        self.assertEquals({'Accept': 'json'}, e.request_pattern.headers)
        self.assertEquals({'test': 1}, e.request_pattern.query_parameters)
        self.assertEquals({'chocolate': 'chip'}, e.request_pattern.cookies)
        self.assertIsInstance(e.request_pattern.basic_auth_credentials, BasicAuthCredentials)
        self.assertEquals('username', e.request_pattern.basic_auth_credentials.username)
        self.assertEquals('password', e.request_pattern.basic_auth_credentials.password)
        self.assertEquals(False, e.request_pattern.browser_proxy_request)
        self.assertEquals('test3', e.request_pattern.body_as_base64)
        self.assertEquals('test4', e.request_pattern.body)

        # match result
        self.assertEquals(0.75, e.match_result.distance)

    @attr('unit', 'serialization', 'nearmisses')
    def test_near_miss_match_response_serialization(self):
        e = NearMissMatchResponse(near_misses=[NearMissMatch(
            request=NearMissMatchRequest(url='test'),
            request_pattern=NearMissRequestPatternResult(url='test2'),
            match_result=NearMissMatchResult(distance=0.75)
        )])
        serialized = e.get_json_data()
        self.assertDictContainsKeyWithValueType(serialized, 'nearMisses', list)
        near_miss = serialized['nearMisses'][0]
        self.assertDictContainsKeyWithValueType(near_miss, 'request', dict)
        self.assertDictContainsKeyWithValue(near_miss['request'], 'url', 'test')
        self.assertDictContainsKeyWithValueType(near_miss, 'requestPattern', dict)
        self.assertDictContainsKeyWithValue(near_miss['requestPattern'], 'url', 'test2')
        self.assertDictContainsKeyWithValueType(near_miss, 'matchResult', dict)
        self.assertDictContainsKeyWithValue(near_miss['matchResult'], 'distance', 0.75)

    @attr('unit', 'serialization', 'nearmisses')
    def test_near_miss_match_response_deserialization(self):
        serialized = {'nearMisses': [
            {
                'request': {'url': 'test'},
                'requestPattern': {'url': 'test'},
                'matchResult': {'distance': 0.75}
            }
        ]}
        e = NearMissMatchResponse.from_dict(serialized)
        self.assertIsInstance(e, NearMissMatchResponse)
        self.assertIsInstance(e.near_misses, list)
        self.assertEquals(1, len(e.near_misses))
        near_miss = e.near_misses[0]
        self.assertIsInstance(near_miss, NearMissMatch)
        self.assertIsInstance(near_miss.request, NearMissMatchRequest)
        self.assertIsInstance(near_miss.request_pattern, NearMissRequestPatternResult)
        self.assertIsInstance(near_miss.match_result, NearMissMatchResult)



from __future__ import unicode_literals

from wiremock.tests.base import BaseClientTestCase, attr
from wiremock.resources.mappings import BasicAuthCredentials, Mapping, MappingRequest, MappingMeta, MappingResponse, \
    AllMappings, DelayDistribution, DelayDistributionMethods


class MappingsSerializationTests(BaseClientTestCase):

    @attr('unit', 'serialization', 'mappings')
    def test_basic_auth_credentials_serialization(self):
        e = BasicAuthCredentials(
            username='username', password='password'
        )
        serialized = e.get_json_data()
        self.assertDictContainsKeyWithValue(serialized, 'username', 'username')
        self.assertDictContainsKeyWithValue(serialized, 'password', 'password')

    @attr('unit', 'serialization', 'mappings')
    def test_basic_auth_credentials_deserialization(self):
        serialized = {'username': 'username', 'password': 'password'}
        e = BasicAuthCredentials.from_dict(serialized)
        self.assertIsInstance(e, BasicAuthCredentials)
        self.assertEquals('username', e.username)
        self.assertEquals('password', e.password)

    @attr('unit', 'serialization', 'mappings')
    def test_mapping_meta_serialization(self):
        e = MappingMeta(total=1)
        serialized = e.get_json_data()
        self.assertDictContainsKeyWithValue(serialized, 'total', 1)

    @attr('unit', 'serialization', 'mappings')
    def test_mapping_meta_deserialization(self):
        serialized = {'total': 1}
        e = MappingMeta.from_dict(serialized)
        self.assertIsInstance(e, MappingMeta)
        self.assertEquals(1, e.total)

    @attr('unit', 'serialization', 'mappings')
    def test_delay_distribution_serialization(self):
        e = DelayDistribution(
            distribution_type=DelayDistributionMethods.LOG_NORMAL, median=0.1, sigma=0.2, upper=4, lower=3
        )
        serialized = e.get_json_data()
        self.assertDictContainsKeyWithValue(serialized, 'type', 'lognormal')
        self.assertDictContainsKeyWithValue(serialized, 'median', 0.1)
        self.assertDictContainsKeyWithValue(serialized, 'sigma', 0.2)
        self.assertDictContainsKeyWithValue(serialized, 'lower', 3)
        self.assertDictContainsKeyWithValue(serialized, 'upper', 4)

    @attr('unit', 'serialization', 'mappings')
    def test_delay_distribution_deserialization(self):
        serialized = {'type': 'lognormal', 'median': 0.1, 'sigma': 0.2, 'lower': 3, 'upper': 4}
        e = DelayDistribution.from_dict(serialized)
        self.assertIsInstance(e, DelayDistribution)
        self.assertEquals('lognormal', e.distribution_type)
        self.assertEquals(0.1, e.median)
        self.assertEquals(0.2, e.sigma)
        self.assertEquals(3, e.lower)
        self.assertEquals(4, e.upper)

    @attr('unit', 'serialization', 'mappings')
    def test_mapping_request_serialization(self):
        e = MappingRequest(
            method='GET', url='test1', url_path='test2', url_path_pattern='test3', url_pattern='test4',
            basic_auth_credentials=BasicAuthCredentials(username='username', password='password'),
            cookies={'chocolate': 'chip'}, headers={'Accept': 'stuff'}, query_parameters={'param': '1'},
            body_patterns={'test': 'test2'}
        )
        serialized = e.get_json_data()
        self.assertDictContainsKeyWithValue(serialized, 'method', 'GET')
        self.assertDictContainsKeyWithValue(serialized, 'url', 'test1')
        self.assertDictContainsKeyWithValue(serialized, 'urlPath', 'test2')
        self.assertDictContainsKeyWithValue(serialized, 'urlPathPattern', 'test3')
        self.assertDictContainsKeyWithValue(serialized, 'urlPattern', 'test4')
        self.assertDictContainsKeyWithValue(serialized, 'basicAuthCredentials', {'username': 'username', 'password': 'password'})
        self.assertDictContainsKeyWithValue(serialized, 'cookies', {'chocolate': 'chip'})
        self.assertDictContainsKeyWithValue(serialized, 'headers', {'Accept': 'stuff'})
        self.assertDictContainsKeyWithValue(serialized, 'queryParameters', {'param': '1'})
        self.assertDictContainsKeyWithValue(serialized, 'bodyPatterns', {'test': 'test2'})

    @attr('unit', 'serialization', 'mappings')
    def test_mapping_request_deserialization(self):
        serialized = {
            'method': 'GET', 'url': 'test1', 'urlPath': 'test2', 'urlPathPattern': 'test3',
            'urlPattern': 'test4',
            'basicAuthCredentials': {'username': 'username', 'password': 'password'},
            'cookies': {'chocolate': 'chip'}, 'headers': {'Accept': 'stuff'},
            'queryParameters': {'param': '1'}, 'bodyPatterns': {'test': 'test2'}
        }
        e = MappingRequest.from_dict(serialized)
        self.assertIsInstance(e, MappingRequest)
        self.assertEquals('GET', e.method)
        self.assertEquals('test1', e.url)
        self.assertEquals('test2', e.url_path)
        self.assertEquals('test3', e.url_path_pattern)
        self.assertEquals('test4', e.url_pattern)
        self.assertIsInstance(e.basic_auth_credentials, BasicAuthCredentials)
        self.assertEquals('username', e.basic_auth_credentials.username)
        self.assertEquals('password', e.basic_auth_credentials.password)
        self.assertEquals({'chocolate': 'chip'}, e.cookies)
        self.assertEquals({'Accept': 'stuff'}, e.headers)
        self.assertEquals({'param': '1'}, e.query_parameters)
        self.assertEquals({'test': 'test2'}, e.body_patterns)

    @attr('unit', 'serialization', 'mappings')
    def test_mapping_response_serialization(self):
        e = MappingResponse(
            additional_proxy_request_headers={'test': '1'}, base64_body='test2', body='test3', body_file_name='test4',
            json_body='test5',
            delay_distribution=DelayDistribution(distribution_type='lognormal', sigma=0.1, median=0.2),
            fault='test6', fixed_delay_milliseconds=500, from_configured_stub='test7', headers={'test': '1'},
            proxy_base_url='test8', status=200, status_message='test9', transformer_parameters={'test2': '2'},
            transformers=['test10']
        )
        serialized = e.get_json_data()
        self.assertDictContainsKeyWithValue(serialized, 'additionalProxyRequestHeaders', {'test': '1'})
        self.assertDictContainsKeyWithValue(serialized, 'base64Body', 'test2')
        self.assertDictContainsKeyWithValue(serialized, 'body', 'test3')
        self.assertDictContainsKeyWithValue(serialized, 'bodyFileName', 'test4')
        self.assertDictContainsKeyWithValue(serialized, 'jsonBody', 'test5')
        self.assertDictContainsKeyWithValue(serialized, 'delayDistribution', {'type': 'lognormal', 'sigma': 0.1, 'median': 0.2})
        self.assertDictContainsKeyWithValue(serialized, 'fault', 'test6')
        self.assertDictContainsKeyWithValue(serialized, 'fixedDelayMilliseconds', 500)
        self.assertDictContainsKeyWithValue(serialized, 'fromConfiguredStub', 'test7')
        self.assertDictContainsKeyWithValue(serialized, 'headers', {'test': '1'})
        self.assertDictContainsKeyWithValue(serialized, 'proxyBaseUrl', 'test8')
        self.assertDictContainsKeyWithValue(serialized, 'status', 200)
        self.assertDictContainsKeyWithValue(serialized, 'statusMessage', 'test9')
        self.assertDictContainsKeyWithValue(serialized, 'transformerParameters', {'test2': '2'})
        self.assertDictContainsKeyWithValue(serialized, 'transformers', ['test10'])

    @attr('unit', 'serialization', 'mappings')
    def test_mapping_response_deserialization(self):
        serialized = {
            'additionalProxyRequestHeaders': {'test': '1'}, 'base64Body': 'test2', 'body': 'test3',
            'bodyFileName': 'test4', 'jsonBody': 'test5',
            'delayDistribution': {'type': 'lognormal', 'sigma': 0.1, 'median': 0.2}, 'fault': 'test6',
            'fixedDelayMilliseconds': 500, 'fromConfiguredStub': 'test7', 'headers': {'test': '1'},
            'proxyBaseUrl': 'test8', 'status': 200, 'statusMessage': 'test9', 'transformerParameters': {'test2': '2'},
            'transformers': ['test10']
        }
        e = MappingResponse.from_dict(serialized)
        self.assertIsInstance(e, MappingResponse)
        self.assertEquals({'test': '1'}, e.additional_proxy_request_headers)
        self.assertEquals('test2', e.base64_body)
        self.assertEquals('test3', e.body)
        self.assertEquals('test4', e.body_file_name)
        self.assertEquals('test5', e.json_body)
        self.assertIsInstance(e.delay_distribution, DelayDistribution)
        self.assertEquals('lognormal', e.delay_distribution.distribution_type)
        self.assertEquals('test6', e.fault)
        self.assertEquals(500, e.fixed_delay_milliseconds)
        self.assertEquals('test7', e.from_configured_stub)
        self.assertEquals({'test': '1'}, e.headers)
        self.assertEquals('test8', e.proxy_base_url)
        self.assertEquals(200, e.status)
        self.assertEquals('test9', e.status_message)
        self.assertEquals({'test2': '2'}, e.transformer_parameters)
        self.assertEquals(['test10'], e.transformers)

    @attr('unit', 'serialization', 'mappings')
    def test_mapping_serialization(self):
        e = Mapping(
            priority=1, request=MappingRequest(method='GET', url='test'),
            response=MappingResponse(status=200, status_message='test2'),
            persistent=False, post_serve_actions={'test': '1'},
            new_scenario_state='test3', required_scenario_state='test4',
            scenario_name='test5'
        )
        serialized = e.get_json_data()
        self.assertDictContainsKeyWithValue(serialized, 'priority', 1)
        self.assertDictContainsKeyWithValue(serialized, 'request', {'method': 'GET', 'url': 'test'})
        self.assertDictContainsKeyWithValue(serialized, 'response', {'status': 200, 'statusMessage': 'test2'})
        self.assertDictContainsKeyWithValue(serialized, 'persistent', False)
        self.assertDictContainsKeyWithValue(serialized, 'postServeActions', {'test': '1'})
        self.assertDictContainsKeyWithValue(serialized, 'newScenarioState', 'test3')
        self.assertDictContainsKeyWithValue(serialized, 'requiredScenarioState', 'test4')
        self.assertDictContainsKeyWithValue(serialized, 'scenarioName', 'test5')

    @attr('unit', 'serialization', 'mappings')
    def test_mapping_deserialization(self):
        serialized = {
            'priority': 1, 'request': {'method': 'GET', 'url': 'test'},
            'response': {'status': 200, 'statusMessage': 'test2'}, 'persistent': False,
            'postServeActions': {'test': '1'}, 'newScenarioState': 'test3',
            'requiredScenarioState': 'test4', 'scenarioName': 'test5'
        }
        e = Mapping.from_dict(serialized)
        self.assertIsInstance(e, Mapping)
        self.assertEquals(1, e.priority)
        self.assertIsInstance(e.request, MappingRequest)
        self.assertEquals('GET', e.request.method)
        self.assertEquals('test', e.request.url)
        self.assertIsInstance(e.response, MappingResponse)
        self.assertEquals(200, e.response.status)
        self.assertEquals('test2', e.response.status_message)
        self.assertEquals(False, e.persistent)
        self.assertEquals({'test': '1'}, e.post_serve_actions)
        self.assertEquals('test3', e.new_scenario_state)
        self.assertEquals('test4', e.required_scenario_state)
        self.assertEquals('test5', e.scenario_name)

    @attr('unit', 'serialization', 'mappings')
    def test_all_mappings_serialization(self):
        e = AllMappings(
            mappings=[Mapping(priority=1), ], meta=MappingMeta(total=1)
        )
        serialized = e.get_json_data()
        self.assertDictContainsKeyWithValue(serialized, 'mappings', [{'priority': 1}, ])
        self.assertDictContainsKeyWithValue(serialized, 'meta', {'total': 1})

    @attr('unit', 'serialization', 'mappings')
    def test_all_mappings_deserialization(self):
        serialized = {
            'mappings': [{'priority': 1}, ],
            'meta': {'total': 1}
        }
        e = AllMappings.from_dict(serialized)
        self.assertIsInstance(e, AllMappings)
        self.assertIsInstance(e.mappings, list)
        m = e.mappings[0]
        self.assertIsInstance(m, Mapping)
        self.assertEquals(1, m.priority)
        self.assertIsInstance(e.meta, MappingMeta)
        self.assertEquals(1, e.meta.total)



from __future__ import unicode_literals

import responses

from wiremock.resources.mappings import HttpMethods
from wiremock.tests.base import BaseClientTestCase, attr
from wiremock.client import NearMisses, NearMissMatch, NearMissMatchPatternRequest, NearMissMatchRequest, \
    NearMissMatchResponse, NearMissMatchResult, NearMissRequestPatternResult


class NearMissesResourceTests(BaseClientTestCase):

    @attr('unit', 'nearmisses', 'resource')
    @responses.activate
    def test_find_nearest_misses_by_request(self):
        e = NearMissMatchResponse(
            near_misses=[NearMissMatch(
                request=NearMissMatchRequest(url='test', method='GET'),
                request_pattern=NearMissRequestPatternResult(url='test1', method='GET'),
                match_result=NearMissMatchResult(distance=0.5)
            ), ]
        )
        resp = e.get_json_data()
        responses.add(responses.POST, 'http://localhost/__admin/near-misses/request', json=resp, status=200)

        near_miss_match_request = NearMissMatchRequest(url='test', method=HttpMethods.GET)
        r = NearMisses.find_nearest_misses_by_request(near_miss_match_request)
        self.assertIsInstance(r, NearMissMatchResponse)
        self.assertIsInstance(r.near_misses, list)
        result = r.near_misses[0]
        self.assertIsInstance(result, NearMissMatch)
        self.assertEquals('test', result.request.url)

    @attr('unit', 'nearmisses', 'resource')
    @responses.activate
    def test_find_nearest_misses_by_request_pattern(self):
        e = NearMissMatchResponse(
            near_misses=[NearMissMatch(
                request=NearMissMatchRequest(url='test', method='GET'),
                request_pattern=NearMissRequestPatternResult(url='test1', method='GET'),
                match_result=NearMissMatchResult(distance=0.5)
            ), ]
        )
        resp = e.get_json_data()
        responses.add(responses.POST, 'http://localhost/__admin/near-misses/request-pattern', json=resp, status=200)

        near_miss_match_request_pattern = NearMissMatchPatternRequest(url='test', method=HttpMethods.GET)
        r = NearMisses.find_nearest_misses_by_request_pattern(near_miss_match_request_pattern)
        self.assertIsInstance(r, NearMissMatchResponse)
        self.assertIsInstance(r.near_misses, list)
        result = r.near_misses[0]
        self.assertIsInstance(result, NearMissMatch)
        self.assertEquals('test', result.request.url)



from __future__ import unicode_literals

from wiremock.constants import make_headers
from wiremock.base.base_resource import BaseResource


class Scenarios(BaseResource):

    @classmethod
    def endpoint(cls):
        return '/scenarios/reset'

    @classmethod
    def endpoint_single(cls):
        return '/scenarios'

    @classmethod
    def entity_class(cls):
        return None

    @classmethod
    def reset_all_scenarios(cls, parameters={}):
        response = cls.REST_CLIENT.post(cls.get_base_uri(cls.endpoint()), headers=make_headers(), params=parameters)
        return cls.REST_CLIENT.handle_response(response)


__all__ = ['Scenarios']



# -*- coding: utf-8 -*-
"""WireMock Server Management."""
import atexit
import socket

import time
from pkg_resources import resource_filename
from subprocess import Popen, PIPE, STDOUT

from six import text_type

from wiremock.server.exceptions import (
    WireMockServerAlreadyStartedError,
    WireMockServerNotStartedError
)


class WireMockServer(object):

    DEFAULT_JAVA = 'java'  # Assume java in PATH
    DEFAULT_JAR = resource_filename('wiremock', 'server/wiremock-standalone-2.6.0.jar')

    def __init__(self, java_path=DEFAULT_JAVA, jar_path=DEFAULT_JAR):
        self.java_path = java_path
        self.jar_path = jar_path
        self.port = self._get_free_port()
        self.__subprocess = None
        self.__running = False

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, type, value, traceback):
        self.stop()

    @property
    def is_running(self):
        return self.__running

    def start(self):
        if self.is_running:
            raise WireMockServerAlreadyStartedError(
                'WireMockServer already started on port {}'.format(self.port)
            )

        cmd = [self.java_path, '-jar', self.jar_path, '--port', str(self.port)]
        try:
            self.__subprocess = Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=STDOUT)
        except OSError as e:
            raise WireMockServerNotStartedError(str(e))  # Problem with Java

        time.sleep(0.1)
        if self.__subprocess.poll() is not None:
            # Process complete - server not started
            raise WireMockServerNotStartedError("\n".join([
                "returncode: {}".format(self.__subprocess.returncode),
                "stdout:",
                text_type(self.__subprocess.stdout.read())
            ]))

        atexit.register(self.stop, raise_on_error=False)
        self.__running = True

    def stop(self, raise_on_error=True):
        try:
            self.__subprocess.kill()
        except AttributeError:
            if raise_on_error:
                raise WireMockServerNotStartedError()

    def _get_free_port(self):
        s = socket.socket(socket.AF_INET, type=socket.SOCK_STREAM)
        s.bind(('localhost', 0))
        address, port = s.getsockname()
        s.close()
        return port


from __future__ import unicode_literals
from calendar import timegm
from copy import deepcopy
import logging

from wiremock import __version__
from wiremock._compat import *


logger = logging.getLogger('wiremock')


class Singleton(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]


DEFAULT_TIMEOUT = 30
DEFAULT_BASE_URL = 'http://localhost/__admin'
USER_AGENT = 'python_wiremock/%s'.format(__version__)
DEFAULT_HEADERS = {'Accept': 'application/json', 'Content-Type': 'application/json', 'user-agent': USER_AGENT}


@add_metaclass(Singleton)
class Config(object):
    timeout = DEFAULT_TIMEOUT
    base_url = DEFAULT_BASE_URL
    user_agent = USER_AGENT
    headers = DEFAULT_HEADERS

Config()  # pre-call once


def make_headers(**kwargs):
    headers = deepcopy(Config.headers)
    for key, value in kwargs.items():
        headers[key] = value
    return headers


def datetime_to_ms(dt):
    if isinstance(dt, integer_types):
        return dt
    else:
        tmp = timegm(date.utctimetuple())
        tmp += float(date.microsecond) / 1000000.0
        return long_(tmp * 1000.0)


__all__ = ['Config', 'make_headers', 'logger', 'datetime_to_ms']



from __future__ import unicode_literals

import responses

from wiremock.resources.near_misses import NearMissMatchRequest
from wiremock.tests.base import BaseClientTestCase, attr
from wiremock.client import RequestResponseDefinition, RequestResponseRequest, RequestResponse, RequestCountResponse, \
    RequestResponseAll, RequestResponseFindResponse, Requests, RequestResponseAllMeta, NearMissMatchPatternRequest, \
    NearMissMatchResponse, NearMissMatch


class RequestsResourceTests(BaseClientTestCase):

    @attr('unit', 'requests', 'resource')
    @responses.activate
    def test_get_all_received_requests(self):
        e = RequestResponseAll(
            requests=[],
            meta=RequestResponseAllMeta(total=1),
            request_journal_disabled=False
        )
        resp = e.get_json_data()
        responses.add(responses.GET, 'http://localhost/__admin/requests', json=resp, status=200)

        r = Requests.get_all_received_requests()
        self.assertIsInstance(r, RequestResponseAll)
        self.assertEquals(False, r.request_journal_disabled)

    @attr('unit', 'requests', 'resource')
    @responses.activate
    def test_get_request(self):
        e = RequestResponse(
            id='1234-5678',
            request=RequestResponseRequest(url='test', method='GET'),
            response_definition=RequestResponseDefinition(url='test', method='GET'),
        )
        resp = e.get_json_data()
        responses.add(responses.GET, 'http://localhost/__admin/requests/1234-5678', json=resp, status=200)

        r = Requests.get_request('1234-5678')
        self.assertIsInstance(r, RequestResponse)
        self.assertEquals('test', r.request.url)
        self.assertEquals('1234-5678', r.id)

    @attr('unit', 'requests', 'resource')
    @responses.activate
    def test_reset_request_journal(self):
        responses.add(responses.POST, 'http://localhost/__admin/requests/reset', body='', status=200)

        r = Requests.reset_request_journal()
        self.assertEquals(200, r.status_code)

    @attr('unit', 'requests', 'resource')
    @responses.activate
    def test_get_matching_request_count(self):
        resp = RequestCountResponse(count=4).get_json_data()
        responses.add(responses.POST, 'http://localhost/__admin/requests/count', json=resp, status=200)

        request = NearMissMatchPatternRequest(url='test', method='GET')

        r = Requests.get_matching_request_count(request)
        self.assertIsInstance(r, RequestCountResponse)
        self.assertEquals(4, r.count)

    @attr('unit', 'requests', 'resource')
    @responses.activate
    def test_get_matching_requests(self):
        e = RequestResponseFindResponse(
            requests=[RequestResponseRequest(method='GET', url='test'), ],
        )
        resp = e.get_json_data()
        responses.add(responses.POST, 'http://localhost/__admin/requests/find', json=resp, status=200)

        request = NearMissMatchPatternRequest(url='test', method='GET')

        r = Requests.get_matching_requests(request)
        self.assertIsInstance(r, RequestResponseFindResponse)
        self.assertIsInstance(r.requests, list)
        self.assertEquals(1, len(r.requests))
        result = r.requests[0]
        self.assertIsInstance(result, RequestResponseRequest)
        self.assertEquals('GET', result.method)
        self.assertEquals('test', result.url)

    @attr('unit', 'requests', 'resource')
    @responses.activate
    def test_get_unmatched_requests(self):
        e = RequestResponseFindResponse(
            requests=[RequestResponseRequest(method='GET', url='test'), ],
        )
        resp = e.get_json_data()
        responses.add(responses.GET, 'http://localhost/__admin/requests/unmatched', json=resp, status=200)

        r = Requests.get_unmatched_requests()
        self.assertIsInstance(r, RequestResponseFindResponse)
        self.assertIsInstance(r.requests, list)
        self.assertEquals(1, len(r.requests))
        result = r.requests[0]
        self.assertIsInstance(result, RequestResponseRequest)
        self.assertEquals('GET', result.method)
        self.assertEquals('test', result.url)

    @attr('unit', 'requests', 'resource')
    @responses.activate
    def test_get_unmatched_requests_near_misses(self):
        e = NearMissMatchResponse(
            near_misses=[NearMissMatch(
                request=NearMissMatchRequest(url='test', method='GET')
            ), ]
        )
        resp = e.get_json_data()
        responses.add(responses.GET, 'http://localhost/__admin/requests/unmatched/near-misses', json=resp, status=200)

        r = Requests.get_unmatched_requests_near_misses()
        self.assertIsInstance(r, NearMissMatchResponse)
        result = r.near_misses[0]
        self.assertIsInstance(result, NearMissMatch)
        self.assertEquals('test', result.request.url)
        self.assertEquals('GET', result.request.method)



from __future__ import unicode_literals

import responses

from wiremock.tests.base import BaseClientTestCase, attr
from wiremock.client import Mapping, MappingMeta, MappingRequest, MappingResponse, Mappings, AllMappings


class MappingsResourceTests(BaseClientTestCase):

    @attr('unit', 'mappings', 'resource')
    @responses.activate
    def test_create_mapping(self):
        e = MappingResponse(
            body='test',
            status=200
        )
        resp = e.get_json_data()
        responses.add(responses.POST, 'http://localhost/__admin/mappings', json=resp, status=200)

        m = Mapping(
            priority=1,
            request=MappingRequest(url='test', method='GET'),
            response=MappingResponse(status=200, body='test')
        )

        r = Mappings.create_mapping(m)
        self.assertIsInstance(r, MappingResponse)
        self.assertEquals(200, r.status)
        self.assertEquals('test', r.body)

    @attr('unit', 'mappings', 'resource')
    @responses.activate
    def test_retrieve_all_mappings(self):
        e = AllMappings(
            mappings=[Mapping(id='1234-5678', priority=1), ],
            meta=MappingMeta(total=1)
        )
        resp = e.get_json_data()
        responses.add(responses.GET, 'http://localhost/__admin/mappings', json=resp, status=200)

        r = Mappings.retrieve_all_mappings()
        self.assertIsInstance(r, AllMappings)
        self.assertIsInstance(r.meta, MappingMeta)
        self.assertEquals(1, r.meta.total)

    @attr('unit', 'mappings', 'resource')
    @responses.activate
    def test_retrieve_mapping(self):
        e = Mapping(
            id='1234-5678', priority=1
        )
        resp = e.get_json_data()
        responses.add(responses.GET, 'http://localhost/__admin/mappings/1234-5678', json=resp, status=200)

        r = Mappings.retrieve_mapping(e)
        self.assertIsInstance(r, Mapping)
        self.assertEquals('1234-5678', r.id)
        self.assertEquals(1, r.priority)

    @attr('unit', 'mappings', 'resource')
    @responses.activate
    def test_update_mapping(self):
        e = Mapping(
            id='1234-5678', priority=1
        )
        resp = e.get_json_data()
        responses.add(responses.PUT, 'http://localhost/__admin/mappings/1234-5678', json=resp, status=200)

        r = Mappings.update_mapping(e)
        self.assertIsInstance(r, Mapping)
        self.assertEquals('1234-5678', r.id)
        self.assertEquals(1, r.priority)

    @attr('unit', 'mappings', 'resource')
    @responses.activate
    def test_persist_mappings(self):
        responses.add(responses.POST, 'http://localhost/__admin/mappings/save', body='', status=200)

        r = Mappings.persist_mappings()
        self.assertEquals(200, r.status_code)

    @attr('unit', 'mappings', 'resource')
    @responses.activate
    def test_reset_mappings(self):
        responses.add(responses.POST, 'http://localhost/__admin/mappings/reset', body='', status=200)

        r = Mappings.reset_mappings()
        self.assertEquals(200, r.status_code)

    @attr('unit', 'mappings', 'resource')
    @responses.activate
    def test_delete_all_mappings(self):
        responses.add(responses.DELETE, 'http://localhost/__admin/mappings', body='', status=200)

        r = Mappings.delete_all_mappings()
        self.assertEquals(200, r.status_code)

    @attr('unit', 'mappings', 'resource')
    @responses.activate
    def test_delete_mapping(self):
        e = Mapping(
            id='1234-5678', priority=1
        )
        responses.add(responses.DELETE, 'http://localhost/__admin/mappings/1234-5678', body='', status=200)

        r = Mappings.delete_mapping(e)
        self.assertEquals(200, r.status_code)



from .api_exception import ApiException
from .api_unavailable_exception import ApiUnavailableException
from .client_exception import ClientException
from .forbidden_exception import ForbiddenException
from .invalid_input_exception import InvalidInputException
from .not_found_exception import NotFoundException
from .requires_login_exception import RequiresLoginException
from .server_exception import ServerException
from .timeout_exception import TimeoutException
from .unexpected_response_exception import UnexpectedResponseException



import sys
from setuptools import setup, find_packages

#next time:
#python setup.py register
#python setup.py sdist upload

version = open('wiremock/VERSION', 'r').readline().strip()
develop_requires = [
    'Sphinx==1.5.3',
    'coverage==4.3.4',
    'detox==0.10.0',
    'mock==2.0.0',
    'nose==1.3.7',
    'python-coveralls==2.9.0',
    'responses==0.5.1',
    'requests==2.13.0',
    'six>=1.10.0',
    'sphinx-rtd-theme==0.2.4',
    'tox==2.6.0',
    'watchdog==0.8.3',
    'wheel>=0.24.0']

long_desc = """
wiremock is an API Client to the Admin API for WireMock Standalone installation: https://wiremock.org/docs

`Documentation <https://wiremock.readthedocs.org/en/latest/>`_

`Report a Bug <https://bitbucket.org/wellaware/python_wiremock/issues>`_
"""

setup(
    name='wiremock',
    version=version,
    description='Wiremock Admin API Client',
    dependency_links=['https://bitbucket.org/wellaware/python_wiremock/archive/{0}.tar.gz#egg=wiremock-{0}'.format(version)],
    long_description=long_desc,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Environment :: Other Environment",
        "Environment :: Plugins",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Natural Language :: English",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: Implementation",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords='wiremock',
    install_requires=[
        'setuptools>=35.0.1',
        'six>=1.10.0',
        'requests==2.13.0'
    ],
    extras_require={
        'develop': develop_requires,
        'docs': ['Sphinx>=1.5.3', 'sphinx-rtd-theme>=0.2.4', 'watchdog>=0.8.3'],
    },
    test_suite='nose.collector',
    tests_require=develop_requires,
    author='Cody Lee',
    author_email='codylee@wellaware.us',
    maintainer='Cody Lee',
    maintainer_email='codylee@wellaware.us',
    url='https://bitbucket.org/wellaware/python_wiremock',
    license='Apache Software License 2.0',
    packages=find_packages(),
    include_package_data=True,
)



from __future__ import unicode_literals

import responses

from wiremock.tests.base import BaseClientTestCase, attr
from wiremock.client import GlobalSetting, GlobalSettings


class SettingsResourceTests(BaseClientTestCase):

    @attr('unit', 'settings', 'resource')
    @responses.activate
    def test_update_settings(self):
        e = GlobalSetting(fixed_delay=500)
        resp = e.get_json_data()
        responses.add(responses.POST, 'http://localhost/__admin/settings', json=resp, status=200)

        r = GlobalSettings.update_global_settings(e)
        self.assertIsInstance(r, GlobalSetting)
        self.assertEquals(500, r.fixed_delay)



# -*- coding: utf-8 -*-
"""WireMockServer Exceptions."""


class WireMockServerException(Exception):
    pass


class WireMockServerAlreadyStartedError(WireMockServerException):
    pass


class WireMockServerNotStartedError(WireMockServerException):
    pass
